# Define directory to read Ground Motions

# GM_file = r'C:\Users\geral\Documents\Research\Openseepy\Nasser_Marafi\SRCSWArchetypes-master\GroundMotions\M9\Realizations\Original\SortedEQFile_('+str(GM_name)+').dat'


# Create NMFS structural analysis model
import numpy as np
import matplotlib.pyplot as plt
import openseespy.opensees as op
import csv
from scipy.optimize import fsolve
from NMFS_model_functions3 import (read_ground_motion,
                                        get_modal_analysis, get_th_analysis,
                                        get_pushover, plot_displacements, plot_roof_drift,
                                        plot_story_shear, plot_hysteresis,
                                        plot_story_disp, plot_interstory_drift,
                                        plot_interstory_drift_all, plot_floor_acceleration,
                                        plot_floor_acceleration_all, plot_spring_response)
# NMFS_model_functions3
# NMFS_VGH_model_functions
import openseespy.postprocessing.Get_Rendering as plot
from NMFS_VGH_model_functionsXY import (get_yield_ratio)




# =============================================================================
# Units
# =============================================================================

m = 1
kg = 1
N = 1
Pa = 1
g = 9.81
inches = 0.0254*m
ft = 12*inches
kip = 4.45*10**3*N
Ksi = 6.89*10**6*Pa
seg = 1
psf = 4.8824*kg/m/m


# =============================================================================
# Building Information
# =============================================================================

numStory = 12
height = 10*ft # height per story
FloorArea = 100*100*ft*ft/2
m_ratio = (130 + 0.5*50)*psf
wall_mass = 35339.42*kg
mass = FloorArea * m_ratio + wall_mass # 50*10**3*kg # (800*2)*453.59237*kg # Mass per floor (10% wall mass)
# Fundamental Period Ta
Ct = 0.02
x = 0.75
H = numStory*height/ft
Ta = 1.4 * Ct * H ** x 


# =============================================================================
# Material Definition
# =============================================================================

mat_type = 'Hysteretic' # 'Bilin' or 'ModIMKPinching' or 'Hysteretic'

if mat_type == 'Bilin':
    # Modified Ibarra-Medina-Krawinkler Deterioration Model with Bilinear Hysteretic Response (Bilin Material)
    K0 = 250*10**6*N/m
    as_Plus = 0.005 # strain hardening ratio for positive loading direction
    Vy = 2800000*N # effective yield strength for positive loading direction
    lamba = 0.0 # Cyclic deterioration parameter
    c_s = 1.0 # rate of strength deterioration
    theta_p = 0.2*m # pre-capping rotation
    theta_pc = 0.2*m # post-capping rotation
    res_Pos = 0.15 # residual strength ratio
    theta_u = 1*m # ultimate rotation capacity
    det = 1. # rate of cyclic deterioration
    
    mat_prop = {}
        
    for i in range(1, numStory+1):
        mat_prop["element"+str(i)] = []
    
    for i in range(1, numStory+1):
        #               K0,     as_Plus, as_Neg, My_Plus,   My_Neg,  Lamda_S, Lamda_C, Lamda_A, Lamda_K, c_S, c_C, c_A, c_K, theta_p_Plus, theta_p_Neg, theta_pc_Plus, theta_pc_Neg, Res_Pos, Res_Neg, theta_u_Plus, theta_u_Neg, D_Plus, D_Neg, nFactor=0.0)
        mat_prop["element"+str(i)] = [K0, as_Plus, as_Plus, Vy, -Vy, lamba, lamba, lamba, lamba, c_s, c_s, c_s, c_s,
                                      theta_p, theta_p, theta_pc, theta_pc, res_Pos, res_Pos, theta_u, theta_u, det, det, 0]
        
elif mat_type == 'ModIMKPinching':
    # Modified Ibarra-Medina-Krawinkler Deterioration Model with Pinched Hysteretic Response (ModIMKPinching Material)
    # wn = 2 * np.pi / Ta
    K0 = 250*10**6*N/m # wn ** 2 * mass # Elastic stiffness units N/m
    as_Plus = 0.005 # strain hardening ratio for positive loading direction
    Vy = 2800000*N # effective yield strength for positive loading direction
    F_pr = 1. # Ratio of the force at which reloading begins to force corresponding to the maximum historic deformation demand (positive loading direction)
    A_pin = 1. # Ratio of reloading stiffness
    lambda_SCA = 20 # Cyclic deterioration parameter for Strength, post-Capping strength,Accelerated reloading stiffness  deterioration
    lambda_K = 10 # Cyclic deterioration parameter for unloading stiffness deterioration
    c_SCAK = 1. # Rate of deterioration
    res = 0.15 # residual strength ratio
    det = 1 # rate of cyclic deterioration
    # Dy = Vy / K0 # Yield displacement units m
    # ductility = 4 
    theta_p = 0.2*m # Dy * ductility - Dy # pre-capping rotation
    # ult = Vy + K0 * Vy * theta_p # What is this ? Ask Preetish
    # Pc = 0.1
    theta_pc = 0.2*m # ult / (K0 * Pc) # post-capping rotation
    theta_u = 1*m # (Dy + theta_p + theta_pc) * 2 # ultimate rotation capacity
    
    mat_prop = {}
    
    for i in range(1, numStory+1):
        mat_prop["element"+str(i)] = []
    
    for i in range(1, numStory+1):
        #           K0, as_Plus, as_Neg, My_Plus, My_Neg, FprPos, FprNeg, A_pinch, Lamda_S, Lamda_C, Lamda_A, Lamda_K, c_S, c_C, c_A, c_K, theta_p_Plus, theta_p_Neg, theta_pc_Plus, theta_pc_Neg, Res_Pos, Res_Neg, theta_u_Plus, theta_u_Neg, D_Plus, D_Neg)
        mat_prop["element"+str(i)] = [K0, as_Plus, as_Plus, Vy, -Vy, F_pr, F_pr, A_pin, lambda_SCA, lambda_SCA, lambda_SCA, lambda_K, c_SCAK, c_SCAK,
                                      c_SCAK, c_SCAK, theta_p, theta_p, theta_pc, theta_pc, res, res, theta_u, theta_u, det, det]
        
elif mat_type == 'Hysteretic':
   
    #From Nasser's model
    Period = [1.8077878267762821,
             0.3064607014835906,
             0.11950217268664291,
             0.06813117767279299,
             0.0465586914263157,
             0.035436649665463976,
             0.028961669501492364,
             0.02490434569465375,
             0.02225816142145726,
             0.020518420826105402,
             0.01941523330422012,
             0.018801941719460223]
    T1 = Period[0] # 18077*seg # Ta # numStory/N_hazus*Te
    # T2= 0.27*T1
    T2 = Period[1] # or 0.27*T1

# =============================================================================
# 1. Calibration of the Elastic Parameters
# =============================================================================
    
    # Solve equations to get γn as a function of α0
    
    α0 = 1.0; error = 1.0 # Initial values
    
    while error > 0.000001:

        α0_guess = α0
        func = lambda γn : 2 + (2 + α0_guess**4/((γn**2)*(γn**2+α0_guess**2)))*np.cos(γn)*np.cosh((γn**2+α0_guess**2)**0.5) + (α0_guess**2/(γn*(γn**2+α0_guess**2)**0.5))*np.sin(γn)*np.sinh((γn**2+α0_guess**2)**0.5)
        
      
        # Use the numerical solver to find the roots
        γn_initial_guess = np.array([0.5, 5.0])
        γ = fsolve(func, γn_initial_guess)
        γ1 = γ[0]
        γ2 = γ[1]
    
    
        # Solve equations to get γn as a function of α0_guess
        func = lambda α0_guess : T2/T1 - (γ1/γ2)*( ((γ1)**2+α0_guess**2)/((γ2)**2+α0_guess**2) )**0.5
        
       
        # Use the numerical solver to find the roots
        α0 = fsolve(func, α0_guess)
        error = abs( (α0 - α0_guess) / α0 )
        
    # # Plot it
    # α0_guess = α0
    # func = lambda γn : 2 + (2 + α0_guess**4/((γn**2)*(γn**2+α0_guess**2)))*np.cos(γn)*np.cosh((γn**2+α0_guess**2)**0.5) + (α0_guess**2/(γn*(γn**2+α0_guess**2)**0.5))*np.sin(γn)*np.sinh((γn**2+α0_guess**2)**0.5)
    # γn1 = np.linspace(0.1, 5, 200)
    # plt.plot(γn1, func(γn1))
    # plt.xlabel("γn")
    # plt.ylabel("expression value")
    # plt.grid()
    # plt.show()
        
    # Plot α0_guess
    # α0_guess = np.linspace(0, 5, 201)
    # plt.plot(α0_guess, func(α0_guess))
    # plt.xlabel("α0")
    # plt.ylabel("expression value")
    # plt.grid()
    # plt.show()
        
        
    # Determine EI & GA
    ang_freq1 = 2*np.pi / T1
    p = mass/height
    H = height*numStory
    EI2 = ang_freq1**2 * p * H**4 /(γ1**2)/(γ1**2 + α0**2)
    GA2 = EI2 * (α0/H)**2



    # From Nasser's model
    Length = 15.* 12.*inches
    Thickness = 24.*inches
    Flange_Thickness = 7.5*12.*inches
    # Ag = ( (180)*(24) )*inches**2 #only core is considered
    Ag = ( (90*24*2)+(180-2*24)*(24) )*inches**2
    I = 24*(180-2*24)**3/12 + 2*(90*24**3/12 + 24*90*(90-12)**2) # in4
    I = I*inches**4
    fpc = 8*Ksi # Units: Pa
    Ec = 4750*np.sqrt(fpc)*1000 # Units: Pa
    # Ec = 57000*np.sqrt(fpc/Ksi*1000)*6894.76 # Units: Pa
    mu = 0.2
    # G = 0.4*Ec # Shear modulus
    G = Ec / (2. * (1. + mu)) # Shear modulus
    Av = 5/6*Ag # Effective shear area
    EI = 0.5*Ec*I # Flexural stiffness /500 - 0.544 / 0.5
    GA = 1.0*G*Av # Shear stiffness /10 - 0.6 / 1.0
    
    # K0 = 223850599.2
    # GA = K0*height*numStory




# =============================================================================
# 2. Calibration of the Yield Parameters
# =============================================================================

    phiMatrix = np.array([[ 0.01328015, -0.11155501,  0.32897093, -0.57703118, -0.83060902,
                             1.        ,  1.        ,  1.        ,  1.        , -0.75830691,
                             0.53282646, -0.27589713],
                           [ 0.04739726, -0.31214588,  0.742494  , -1.        , -1.        ,
                             0.65881475,  0.11028588, -0.41906785, -0.9018704 ,  0.99410573,
                            -0.86433353,  0.50125622],
                           [ 0.0997562 , -0.54138487,  1.        , -0.8437817 , -0.17093696,
                            -0.6263622 , -0.9301887 , -0.68872027,  0.00149773, -0.71404048,
                             1.        , -0.70590519],
                           [ 0.16777988, -0.74397732,  0.95596179, -0.15489703,  0.79449651,
                            -0.8712761 ,  0.00679961,  0.84907675,  0.92058183,  0.0238426 ,
                            -0.8732175 ,  0.86085561],
                           [ 0.24893605, -0.87425789,  0.5991235 ,  0.61289584,  0.84470075,
                             0.28813607,  0.94232473,  0.19790719, -0.96029275,  0.67486642,
                             0.52776973, -0.96169038],
                           [ 0.34077182, -0.900166  ,  0.04826942,  0.93506133, -0.06398677,
                             0.99551655, -0.0876805 , -0.96254676,  0.09066836, -1.        ,
                            -0.04705442,  1.        ],
                           [ 0.44095465, -0.80583043, -0.49645565,  0.59525169, -0.90999991,
                             0.11915638, -0.93058901,  0.36894142,  0.8671538 ,  0.76936484,
                            -0.44480164, -0.97374628],
                           [ 0.54731868, -0.59218224, -0.82984375, -0.15646974, -0.74796509,
                            -0.95185421,  0.17826631,  0.7424653 , -0.99786037, -0.10952854,
                             0.82500726,  0.88514905],
                           [ 0.65791507, -0.27531564, -0.81757188, -0.77512916,  0.23119316,
                            -0.52273645,  0.92562454, -0.81474521,  0.18798843, -0.60624289,
                            -0.99238815, -0.73782933],
                           [ 0.77106541,  0.11737025, -0.44328302, -0.79850139,  0.90774444,
                             0.70208138, -0.2350552 , -0.28583093,  0.82073103,  0.99888306,
                             0.91686702,  0.54741175],
                           [ 0.88541712,  0.55293035,  0.19168657, -0.16081261,  0.47164725,
                             0.72251699, -0.83302387,  0.92210086, -0.99503533, -0.79820273,
                            -0.58310317, -0.30892155],
                           [ 1.        ,  1.        ,  0.9177733 ,  0.78521895, -0.70528862,
                            -0.61554986,  0.49105756, -0.41749675,  0.37129149,  0.25904196,
                             0.17212473,  0.08630008]])

    # From Displacement Response Spectrum
    
    RSdisp = 2*0.001* np.array ([1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1]) *m
    ModalParticipation = []
    for mode in range(numStory):
        num = 0
        den = 0
        for story in range(numStory):
            num = num + mass*phiMatrix[story,mode]
            den = den + mass*(phiMatrix[story,mode]**2)
        ModalParticipation.append(num/den)
    
    # Calculate the inter-story displacements    
        
    StoryDisp = np.zeros(shape=(numStory,numStory))
    for mode in range(numStory):
        StoryDisp[:,mode] = ModalParticipation[mode] * phiMatrix[:,mode] * RSdisp[mode]
    InterstoryDisp = StoryDisp
    for story in range(numStory-1):
        InterstoryDisp[story+1,:] = (StoryDisp[story+1,:] - StoryDisp[story,:])
    
    # Calculate the inter-story rotation   
        
    StoryRot = np.zeros(shape=(numStory,numStory))
    for story in range(numStory):
        StoryRot[story,:] = StoryDisp[story,:] / ((story+1)*height) 
    InterstoryRot = StoryRot
    for story in range(numStory-1):
        InterstoryRot[story+1,:] = (StoryRot[story+1,:] - StoryRot[story,:])
    
    # Calculate the inter-story shear/flexural force
    
    InterstoryShear = np.zeros(shape=(numStory,numStory))
    InterstoryMoment = np.zeros(shape=(numStory,numStory))
    InterstoryShear[:,:] = InterstoryDisp[:,:] * GA / height
    InterstoryMoment[:,:] = InterstoryRot[:,:] * EI / height        
    
    # Combine shear forces and bending moments (SRSS)
    
    TotalShear = np.zeros((numStory))
    TotalMoment = np.zeros((numStory))
    for story in range(numStory):
        Va = np.sqrt( sum((InterstoryShear[story,:])**2) )
        Ma = np.sqrt( sum((InterstoryMoment[story,:])**2) )
        TotalShear[story] = Va
        TotalMoment[story] = Ma

    # Set limits to the shear and moment to get DESIGN shear and moment
        
    # Get yield shear and moment - FROM HAZUS NASSERS DATA ?
    # OmegaY = 1.1
    # Vy = OmegaY*TotalShear
    # My = OmegaY*TotalMoment
    
    
    # Trial with Nasser's info
    Cs = 0.08
    OmegaY = 0.675
    Vd = Cs*g*mass*numStory # in Newtons

    rf = np.zeros(numStory)
    rs = np.zeros(numStory)
    for i in range(numStory):
        rf_temp = 1-i*(1-0.6)/(numStory-1) #ratio between the shear strength of the ith story and that of the ground story
        rf[i] = rf_temp
        rs_temp = 1-(i*(i+1))/(numStory*(numStory+1)) #ratio between the shear strength of the ith story and that of the ground story
        rs[i] = rs_temp
    My = 64*1000*1000*rf   #66000 
    # My = 64*1000*1000*np.ones(numStory)
    Vy = 11551900*rs
    # Vy = 11551900*np.ones(numStory)
    # Vy = OmegaY*Vd*rs
    # Vy = OmegaY*Vd*np.ones(numStory)
        
    Dy = Vy*(height-0.0)/(GA)
    Ry = My*(height-0.0)/(EI)

# =============================================================================
# 3. Calibration of the Peak Point
# =============================================================================    

    # Get ultimate shear and moment - FROM HAZUS NASSERS DATA ?
    
    OmegaP = 1.8 # 1.8 overstrength factor given by Table 5.5 of the HAZUS
    Vp = OmegaP*Vy
    Mp = OmegaP*My
    # Dp = Vp*height/GA
    # Rp = Mp*height/EI
    
    method = 'Ductility_factor_method' # 'Stiffness reduction method' or 'Ductility factor method'

    if method == 'Stiffness_reduction_method': # Method 01: Get Stiffness reduction factor 'eta' 
        eta = 0.875 # or 0.80 from ACI 318-08 10.10.4.1
        ErI = eta*EI # reduced bending stiffness
        GrA = eta*GA # reduced shear stiffness
        Dp = Vp*height/GrA
        Rp = Mp*height/ErI

    elif method == 'Ductility_factor_method':
        ductility = 6.0 # 4.75from HAZUS 6.0
        Dp = ductility*Dy
        Rp = ductility*Ry  
        
    # Get ultimate displacement and rotation
    drift_co = 0.04 # inter-story drift ratio at the threshold of the complete damage state (Table 5.9a)
    Du = drift_co*height # Disp. at Threshold of Complete Damage State (Table 6.4c)
    Ru = drift_co

# =============================================================================
# 4. Calibration of the Hysteretic Parameter
# =============================================================================      
    
    # Hysteretic behavior - Pinching model proposed by Steelman and Hajjar (2009)
    k_degradation = 0.6 # 0.6/0.3/0.1 degradation factor for short/mod/long earthquake C1M Low Code (Table 5.18)
    # tau = Ap/Ab
    tau = k_degradation

# =============================================================================
# CREATE MATERIAL
# =============================================================================    

    #My= np.zeros(numStory)
    #Vy= np.zeros(numStory)
    mat_prop = {}
    pinchX = 0.50 # to calibrate 0.5
    pinchY = 0.45 # to calibrate 0.4 /0.45
    damage1 = 0 # ductility-1
    damage2 = 0 # Eii/Eult
    beta = 0.45 # 0.45
    
    for i in range(1, numStory+1):
        mat_prop["Shear_element"+str(i)] = []
    for i in range(1, numStory+1):
        mat_prop["Flexural_element"+str(i)] = []
    
    for i in range(numStory):  # Shear Spring
        p1 = [Vy[i], Dy[i]]
        p2 = [Vp[i], Dp[i]]
        p3 = [Vp[i], Du]
        n1 = [-Vy[i], -Dy[i]]
        n2 = [-Vp[i], -Dp[i]]
        n3 = [-Vp[i], -Du]
        
        #        *p1, *p2, *p3=p2, *n1, *n2, *n3=n2, pinchX, pinchY, damage1, damage2, beta)
        mat_prop["Shear_element"+str(i+1)] = [*p1, *p2, *p3, *n1, *n2, *n3, pinchX, pinchY, damage1, damage2, beta]
        
        # # Backbone Shear Spring Curve Plots
        # sf = 1/1000 # 1/1000 for kN , 0.00022480894387096 for kips
        # plt.plot([0, Dy[i], Dp[i], Du] , [0, Vy[i]*sf, Vp[i]*sf, Vp[i]*sf])
        # # plt.ylim([0, 5000])
        # plt.xlim([0, 0.02])
        # plt.xlabel('Displacement(m)')
        # plt.ylabel('Shear Force (kN)')
        # plt.title('Shear Backbone Curve for Level '+str(i+1))
        # plt.grid()
        # plt.show()
        
    for i in range(numStory):  # Flexural Spring  
        # My = My*height
        # Ry = Ry*height
        
        p1 = [My[i], Ry[i]]
        p2 = [Mp[i], Rp[i]]
        p3 = [Mp[i], Ru]
        n1 = [-My[i], -Ry[i]]
        n2 = [-Mp[i], -Rp[i]]
        n3 = [-Mp[i], -Ru]
       
        #        *p1, *p2, *p3=p2, *n1, *n2, *n3=n2, pinchX, pinchY, damage1, damage2, beta)
        mat_prop["Flexural_element"+str(i+1)] = [*p1, *p2, *p3, *n1, *n2, *n3, pinchX, pinchY, damage1, damage2, beta]
        
        # # Backbone Flexural Spring Curve Plots
        # sf = 1/1000 # 1/1000 for kN , 0.00022480894387096 for kips
        # plt.plot([0, Ry[i], Rp[i], Ru] , [0, My[i]*sf, Mp[i]*sf, Mp[i]*sf])
        # plt.xlim([0, 0.02/height])
        # plt.xlabel('Rotation(rad)')
        # plt.ylabel('Moment (kN-m)')
        # plt.title('Flexural Backbone Curve for Level '+str(i+1))
        # plt.grid()
        # plt.show()
        

else:
    print('No material type definition')




Outputs_list = []
GM_first = 1
GM_last = 30
NumSimulations = (GM_last - GM_first) + 1

for GM_number in range(GM_first, GM_last+1):

    # =============================================================================
    # Read ground motion from csv to list
    # =============================================================================
    
    if GM_number < 10 :
        GM_name = 'csz00'+str(GM_number)+'-A11923-NS' # all data for GM_25
    else :
        GM_name = 'csz0'+str(GM_number)+'-A11923-NS' # all data for GM_25   
    
    
    # GM_file = 'sampleGm.dat'
    GM_file = r'C:\Users\geral\Documents\Research\Openseepy\Nasser_Marafi\SRCSWArchetypes-master\GroundMotions\M9\Realizations\Original\SortedEQFile_('+str(GM_name)+').dat'
    
    scaling_factor = 1.0
    dt = 0.02
    time, motion = read_ground_motion(GM_file, scaling_factor, dt, plot = 0)
    
    # =============================================================================
    # OpenSees Analysis
    # =============================================================================
    
    analysisName = 'Modal Analysis'
    
    frequency, period, modalMatrix, Participation = get_modal_analysis(numStory, height, mass, mat_type, mat_prop, plot = 0)
    
    
    analysisName = 'Time History Analysis'
    
    outputs = get_th_analysis(numStory, height, mass, mat_type, mat_prop, time, motion, dt, xi = 0.02, pDelta = 0)

    # Save outputs for run GM_number
    Outputs_list.append(outputs)

# # =============================================================================
# # Save outputs # section before adding multiple running option
# # =============================================================================

# import pandas as pd

# # Inter-story Drift

# numStory = outputs['numStory']
# height = outputs['height'] -0.2
# Ygrid = []
# Max_Interstory_Drifts = []
# for i in range(1, numStory+1):
#     Ygrid.append(i)
#     Interstory_Drift = (outputs["disp"+str(i)][:] - outputs["disp"+str(i-1)][:]) / height*100
#     Max_Interstory_Drifts.append(max(abs(Interstory_Drift))) 

# df1 = pd.DataFrame(Max_Interstory_Drifts)
# df1.to_csv('NMFS_vs_Archetype/Outputs_to_compare/GM_'+str(GM_number)+'_max_drift_NMFS.csv', index=False)


# # Peak Floor Acceleration

# numStory = outputs['numStory']
# Ygrid = []
# Max_Story_Accel = []
# Max_Story_Accel2 = []
# for i in range(1, numStory+2):
#     Ygrid.append(i-1)
#     Story_Accel = outputs["abs_accel"+str(i-1)][:]
#     Max_Story_Accel.append(max(abs(Story_Accel))) # normalized

# df2 = pd.DataFrame(Max_Story_Accel)
# df2.to_csv('NMFS_vs_Archetype/Outputs_to_compare/GM_'+str(GM_number)+'_max_acc_NMFS.csv', index=False) 


# =============================================================================
# Plots
# =============================================================================

if NumSimulations == 1 :
    
    #plot_displacements(outputs) # Plot Time vs Displacements
    
    plot_roof_drift(outputs, scaling_factor, GM_number) # Plot Time vs Roof Drift
    plot_story_shear(outputs, scaling_factor, GM_number) # Plot Maximum Story Shear
    plot_hysteresis(outputs, scaling_factor, GM_number) # Plot First Inter-Story Drift vs Base Shear
    plot_story_disp(outputs, scaling_factor, GM_number) # Plot Maximum Story Drift
    plot_interstory_drift(outputs, scaling_factor, GM_number) # Plot Maximum Interstory Drift
    plot_floor_acceleration(outputs, scaling_factor, GM_number) # Plot Peak Floor Acceleration
    # plot_spring_response(outputs, mat_prop, backbone_curve = 0)
    
    # #Animation - TH analysis
    # op.wipe()
    # Model = 'NMFS'
    # LoadCase = 'Transient'
    # ani = plot.animate_deformedshape( Model, LoadCase, 2., scale=10)
    # #%matplotlib qt
    # #%matplotlib inline
    
    # analysisName = 'Pushover Analysis'
    
    # Umax = 2.5/100*numStory*height
    # Nsteps = 500
    # nodeControl = numStory+1
    # tol = 1.*10**-8
    # Step, Pplot, Uplot = get_pushover(numStory, height, Umax, Nsteps, nodeControl, tol, modalMatrix, pDelta = 0)
    
    # yield_shear = Vy[0]
    # α = 1 # participation of flexural and shear deformations
    # Ky = get_yield_ratio(α)
    # yield_moment = yield_shear * H / Ky

# =============================================================================
# Plots All
# =============================================================================



if NumSimulations != 1 :
    plt.rcParams["figure.figsize"] = (3.25,2.20)
    # plt.rcParams["figure.figsize"] = plt.rcParamsDefault["figure.figsize"]
    plt.rcParams['axes.linewidth'] = 1.25     
    # plot_story_shear_all(Outputs_list) # Plot Maximum Story Shear
    plot_floor_acceleration_all(Outputs_list, GM_first) # Plot Peak Floor Accelerations
    # plot_story_disp_all(Outputs_list) # Plot Maximum Story Drift
    plot_interstory_drift_all(Outputs_list, GM_first) # Plot Maximum Interstory Drift
    
    # Check simulations 
    print ('-----------------------------------------')
    print ('-----------------------------------------')
    # WARNING MESSAGES
    Total_time = len(time)*dt
    for GM_number in range(NumSimulations):
        time_max = len(Outputs_list[GM_number]['time'])*dt
        GM_ID = GM_number + GM_first
        if time_max == Total_time:
            print (str(GM_ID)+" Run Complete")
        else:
            print (str(GM_ID)+" Run Incomplete, fails at "+str(time_max)+"s")