# Create MDOF structural analysis model
import numpy as np
import matplotlib.pyplot as plt
import openseespy.opensees as op
import csv
import scipy.linalg as la
import openseespy.postprocessing.Get_Rendering as plot


# =============================================================================
# Preliminary Modal Analysis
# =============================================================================
    
def get_eigenvalues(numStory):
    K0 = 1.0 # 28700*1000
    mass = 1.0 # 8353
    # Mass and Stiffness matrixes
    I = np.zeros((numStory, numStory))
    for i in range(numStory):
        I[i, i] = 1
    A = np.zeros((numStory, numStory))
    A[0, 0] = 2; A[0, 1] = -1
    A[numStory-1, numStory-2] = -1; A[numStory-1, numStory-1] = 1
    for i in range(1, numStory-1):
        A[i, i] = 2; A[i, i-1] = -1; A[i, i+1] = -1
    K = K0*A
    M = mass*I
    # Eigenvalue Problem
    evals, evecs = la.eig(K, M)
    mode1 = evecs[:,numStory-1]
    # mode1 = mode1 / max(mode1)
    # Lu paper
    num = np.dot(np.dot(mode1.T, I),mode1)
    den = np.dot(np.dot(mode1.T, A),mode1)
    lambda0 = num/den
       
    return lambda0


# =============================================================================
# Modal Analsys
# =============================================================================
    
def get_modal_analysis(numStory, height, mass, mat_type, mat_prop, plot):

    # remove existing model
    op.wipe()
    
    # set modelbuilder
    op.model('basic', '-ndm', 2, '-ndf', 3)
    
    # Nodes and boundary conditions
    op.node(1,0.,0.); op.fix(1, 1, 1, 1)
    
    for i in range(2, numStory+2):
        op.node(i,0.,height*(i-1))
        op.fix(i, 0., 0., 0.) # no fixed
            
    for i in range(2, numStory+2):
        op.mass(i, mass, 0, 0.) 
    
         
    # Define material        
    for i in range(1, numStory+1):
        # Shear springs material
        shear_mat_tag = i
        mat_property1 = mat_prop["Shear_element"+str(i)]
        op.uniaxialMaterial(mat_type, shear_mat_tag, *mat_property1)
        # Flexural spring material
        flex_mat_tag = (i+numStory)
        mat_property2 = mat_prop["Flexural_element"+str(i)]
        op.uniaxialMaterial(mat_type, flex_mat_tag, *mat_property2)
    
    op.uniaxialMaterial('Elastic', 1+2*numStory, 1.*10**10)
    
    # op.geomTransf('Linear', 1, '-jntOffset', 0,0, 0,0)
    # Define elements
    for i in range(1, numStory+1):
        # Shear springs
        eleTag = i
        bot_node = i
        top_node = i+1
        shear_mat_tag = i
        flex_mat_tag = (i+numStory)
        op.element('twoNodeLink', eleTag, bot_node, top_node, '-mat', 1+2*numStory, shear_mat_tag, flex_mat_tag, "-dir", 1,2,3, '-pDelta', 0.5,0.5 , '-shearDist', 0.5,0.5, '-doRayleigh', 1)  
        # op.element('twoNodeLink', eleTag, bot_node, top_node, '-mat', 1+2*numStory, shear_mat_tag, flex_mat_tag, "-dir", 1,2,3,'-shearDist', 0.5,0.5, '-doRayleigh', 1)  
        #op.element('twoNodeLink', eleTag, bot_node, top_node, '-mat', mat_tag, "-dir", 2, 3, '-doRayleigh', 1)
 
    # OpenSees eigenvalue analysis
    omegaSquared = op.eigen('-fullGenLapack', numStory) 
    
    # Periods and frequencies of the structure
    omega = [i**0.5 for i in omegaSquared]
    frequency = [i/(2*np.pi) for i in omega]
    period = [2*np.pi/i for i in omega]
    
    # Mode Shapes
    modalMatrix = np.zeros(shape=(numStory+1,numStory))
    for i in range(0, numStory+1):
        for j in range(0, numStory):
            modalMatrix[i,j] = op.nodeEigenvector(i+1, j+1, 1)
    h = np.zeros(shape=(numStory+1))
    for i in range(0, numStory+1):
        h[i] = i
    
    # Plot Mode shape n
    if plot == 1:
        for i in range(0, numStory):
            n = i
            plt.plot(modalMatrix[:,n], h, [0, 0], [0, numStory])
            plt.xlabel('Mode '+str(n+1))
            plt.ylabel('Story Level')
            plt.title('Normalized Modal Shape')
            plt.grid()
            plt.show()
    
# Participation Factors

    r = -1*np.ones(numStory)
    M = np.zeros((numStory, numStory))
    for i in range(numStory):
        M[i, i] = 1
    
    phiMatrix = modalMatrix[1:numStory+1,:]
    alpha = (np.dot(np.dot(phiMatrix.T, M), r))
    Participation = (alpha**2)/sum(alpha**2)
       
    return frequency, period, modalMatrix, Participation


# =============================================================================
# Read Ground Motion from csv to list
# =============================================================================

def read_ground_motion(GM_file, scaling_factor,dt, plot):
    time=[]
    # motion=[]
    # with open(GM_file, 'r') as rf:
    #     reader = csv.reader(rf, delimiter=',')
    #     for row in reader:
    #         time.append(float(row[0]))
    #         motion.append(float(row[1])*scaling_factor)
            
    motion=np.loadtxt(GM_file, delimiter='\n')       
    
    for t in range(len(motion)):
        time.append(t*dt)
    
    if plot == 1:
        plt.plot(time,motion)
        plt.xlabel('Time (sec)')
        plt.ylabel('Ground Acceleration (g)')
        plt.title('Earthquake Ground Motion')
        plt.grid()
        plt.show()
    return time, motion


# =============================================================================
# Time History Analysis
# =============================================================================
    
def get_th_analysis(numStory, height, mass, mat_type, mat_prop, time, motion, dt, xi, pDelta):

    # remove existing model
    op.wipe()
    
    # set modelbuilder
    op.model('basic', '-ndm', 2, '-ndf', 3)
    
    # Nodes and boundary conditions
    op.node(1,0.,0.); op.fix(1, 1, 1, 1)
    op.node(1*100,10.,0.); op.fix(1*100, 1, 1, 1) # leaning column 5m apart
    
    for i in range(2, numStory+2):
        op.node(i,0.,height*(i-1))
        op.fix(i, 0., 0., 0.)
        # leaning column 5m apart
        op.node(i*100,10.,height*(i-1))
        op.fix(i*100, 0., 0., 1.)  
        
    for i in range(2, numStory+2):
        op.mass(i, mass, 0., 0.)
        # op.mass(i*100, 0, mass, 0.) # leaning column 5m apart
        
    # Define material        
    for i in range(1, numStory+1):
        # Shear springs material
        shear_mat_tag = i
        mat_property1 = mat_prop["Shear_element"+str(i)]
        op.uniaxialMaterial(mat_type, shear_mat_tag, *mat_property1)
        # Flexural spring material
        flex_mat_tag = (i+numStory)
        mat_property2 = mat_prop["Flexural_element"+str(i)]
        op.uniaxialMaterial(mat_type, flex_mat_tag, *mat_property2)
    
    op.uniaxialMaterial('Elastic', 1+2*numStory, 1.*10**15)
    # op.uniaxialMaterial('Steel02',7000,379e6,202.7e9,0.01,18.5,0.925,0.15) # leaning column 5m apart
    
    op.geomTransf('PDelta', 1, '-jntOffset', 0,0, 0,0)
    # Define elements
    for i in range(1, numStory+1):
        # Shear/ Flexural springs
        eleTag = i
        bot_node = i
        top_node = i+1
        shear_mat_tag = i
        flex_mat_tag = (i+numStory)
        op.element('twoNodeLink', eleTag, bot_node, top_node, '-mat', 1+2*numStory, shear_mat_tag, flex_mat_tag, "-dir", 1,2,3, '-pDelta', 0.5,0.5 , '-shearDist', 0.5, '-doRayleigh', 1)  
        # op.element('twoNodeLink', eleTag, bot_node, top_node, '-mat', 1+2*numStory, shear_mat_tag, flex_mat_tag, "-dir", 1,2,3,'-shearDist', 0.5,0.5, '-doRayleigh', 1)  
        # op.element('twoNodeLink', eleTag, bot_node, top_node, '-mat', mat_tag, "-dir", 2, 3, '-doRayleigh', 1)
        
        eleTag_lc = i*100
        lc_mat_tag = (1+2*numStory)
        op.element('truss', eleTag_lc, bot_node*100, top_node*100, 223.53e-4, lc_mat_tag) # leaning column 5m apart
                
        # Rigid links
        left_node = top_node
        right_node = top_node*100
        op.equalDOF(left_node, right_node, 1)
        
    # Damping ratios for modes
    zeta1 = xi
    zeta2 = xi
    
    # OpenSees eigenvalue analysis
    omegaSquared = op.eigen('-fullGenLapack', numStory) 
    omega1 = omegaSquared[0]**0.5
    omega2 = omegaSquared[1]**0.5
    
    # Compute Rayleigh damping coefficients
    A = np.zeros(shape=(2,2))
    A[0,0] = 1/omega1; A[0,1] = omega1
    A[1,0] = 1/omega2; A[1,1] = omega2
    
    b = np.zeros(2)
    b[0] = 2*zeta1
    b[1] = 2*zeta2
    
    # Solve linear system Ax=b
    x = np.linalg.solve(A,b)
    
    #             M    KT  KI  Kn
    op.rayleigh(x[0],0.0,0.0,x[1])
    
    if pDelta == 1:

        # define GRAVITY ANALYSIS -------------------------------------------------------------
        op.timeSeries('Linear', 1)
        op.pattern('Plain', 1, 1,)
        if plot == 1:
            plt.plot(time,motion)
            plt.xlabel('Time (sec)')
            plt.ylabel('Ground Acceleration (g)')
            plt.title('Earthquake Ground Motion')
            plt.grid()
            plt.show()
        for i in range(2, numStory+2):
            op.load(i, 0., -3448529.8709264626*0.367, 0.) # wall
            # op.load(i*100, 0., -3448529.8709264626*0.633, 0.) # leaning column
        
        op.constraints('Plain')  				# how it handles boundary conditions
        op.numberer('Plain')			    # renumber dof's to minimize band-width (optimization), if you want to
        op.system('BandGeneral')		    # how to store and solve the system of equations in the analysis
        op.algorithm('Newton')                 # use Linear algorithm for linear analysis
        op.integrator('LoadControl', 0.1)			# determine the next time step for an analysis, # apply gravity in 10 steps
        op.analysis('Static')					    # define type of analysis static or transient
        op.analyze(10)					        # perform gravity analysis
        op.loadConst('-time', 0.0)				# hold gravity constant and restart time    
        
        op.wipeAnalysis()
    
    # Define the dynamic analysis
    load_tag_dynamic = 2
    pattern_tag_dynamic = 2
    
    values = [i * -1 for i in motion]  # should be negative ????
    op.timeSeries('Path', load_tag_dynamic, '-dt', dt, '-values', *values)
    op.pattern('UniformExcitation', pattern_tag_dynamic, 1, '-accel', load_tag_dynamic, '-fact', 9.81)
    
    # Animation
    Model = 'NMFS'
    LoadCase = 'Transient'
    plot.createODB(Model, LoadCase, Nmodes = 3, deltaT = 2., recorders = [])
    

    op.algorithm('Newton')
    op.system('SparseGeneral')
    op.numberer('RCM')
    op.constraints('Transformation')
    op.integrator('Newmark',0.5,0.25)
    op.analysis('Transient')
    
    # Analysis time step and number of steps
    tol = 1.0e-8
    iterations = 1000
    op.test('EnergyIncr', tol, iterations, 0, 2)
    Nsteps = len(time)
        
    base_node = 1
    firstStory_node = 2
    roof_node = numStory+1
    outputs = {
        "numStory": numStory,
        "height": height,
        "time": [],
        "rel_disp_1story": [],
        "rel_disp_roof": [],
        "rel_accel": [],
        "rel_vel": [],
        "base_shear": [],
        "base_axial_lc": []
    }
    
    # StoryShears  = {}
    # StoryDisp = {}
    for i in range(1, numStory+1):
            outputs["shear"+str(i)] = []
            outputs["moment"+str(i)] = []
            outputs["rel_dif_disp"+str(i)] = []
    for i in range(1, numStory+2):
            outputs["disp"+str(i-1)] = []
            outputs["abs_accel"+str(i-1)] = []
            
    for k in range(Nsteps):
        ok = op.analyze(1,dt)
        if ok < 0:
            break
        curr_time = op.getTime()
        outputs["time"].append(curr_time)
        outputs["rel_disp_1story"].append(op.nodeDisp(firstStory_node, 1))
        outputs["rel_disp_roof"].append(op.nodeDisp(roof_node, 1))
        outputs["rel_accel"].append(op.nodeAccel(roof_node, 1)/9.8)
        outputs["rel_vel"].append(op.nodeVel(roof_node, 1))
        
        op.reactions()
        outputs["base_shear"].append(-op.nodeReaction(base_node, 1))  # Negative since diff node
        outputs["base_axial_lc"].append(-op.nodeReaction(base_node*100, 2))  # Reaction Leaning Column
        for i in range(1, numStory+1):
            node = i
            outputs["shear"+str(i)].append(-op.eleForce(node, 1))
            outputs["moment"+str(i)].append(-op.eleForce(node, 3))
            outputs["rel_dif_disp"+str(i)].append(op.nodeDisp(node+1, 1)-op.nodeDisp(node, 1))
        for i in range(1, numStory+2):
            node = i
            outputs["disp"+str(i-1)].append(op.nodeDisp(node, 1))
            outputs["abs_accel"+str(i-1)].append(op.nodeAccel(node, 1)/9.8-motion[k]) # (node, 1)/9.8+motion[k])
            
    for item in outputs:
        outputs[item] = np.array(outputs[item])
   
    # with open('NTHA.csv', 'w') as f:
    #     w = csv.DictWriter(f, outputs.keys())
    #     w.writeheader()
    #     w.writerow(outputs)
   
    return outputs

	
# Plots
    
def plot_displacements(outputs): # Plot Time vs Displacements

    plt.plot(outputs['time'], outputs["disp"+str(1)], outputs['time'], outputs["disp"+str(2)], outputs['time'], outputs["disp"+str(3)],
             outputs['time'], outputs["disp"+str(4)], outputs['time'], outputs["disp"+str(5)], outputs['time'], outputs["disp"+str(6)],
             outputs['time'], outputs["disp"+str(7)], outputs['time'], outputs["disp"+str(8)], outputs['time'], outputs["disp"+str(9)],
             outputs['time'], outputs["disp"+str(10)], outputs['time'], outputs["disp"+str(11)], outputs['time'], outputs["disp"+str(12)])
    plt.xlabel('Time (sec)')
    plt.ylabel('Story Displacement (m)')
    plt.title('Story Displacements')
    plt.grid()
    path = r'C:\Users\geral\Documents\Research\_doc\figures\RoofDisp'
    plt.savefig(path, dpi =300)
    plt.show()    
    
def plot_roof_drift(outputs, scaling_factor, GM_number): # Plot Time vs Roof Drift
    
    H = outputs['numStory']*outputs['height']
    
    if scaling_factor == 0.1:            
        time_FE=[]
        roof_disp_FE=[]
        with open(r'C:\Users\geral\Documents\Research\Openseepy\NMFS_vs_Archetype\roof_disp_FE_SC01.csv', 'r') as rf:
            reader = csv.reader(rf, delimiter=',')
            for row in reader:
                time_FE.append(float(row[0]))
                roof_disp_FE.append(float(row[1]))       
                  
    elif scaling_factor == 1.0:
        time_FE=[]
        roof_disp_FE=[]
        # roof_disp_FE2=[]
        # with open(r'C:\Users\geral\Documents\Research\Openseepy\NMFS_vs_Archetype\roof_disp_FE.csv', 'r') as rf:
        with open(r'C:\Users\geral\Documents\Research\Openseepy\Nasser_Marafi\SRCSWArchetypes-master\Outputs_ger\GM_'+str(GM_number)+'_roof_disp.csv', 'r') as rf:
            reader = csv.reader(rf, delimiter=',')
            # next(reader)
            for row in reader:
                # time_FE.append(float(row[0]))
                roof_disp_FE.append(float(row[0]))  
                # roof_disp_FE2.append(float(row[2]))
            for i in range(len(roof_disp_FE)):
                time_FE.append(i*0.02)

    plt.plot(time_FE, roof_disp_FE, linewidth=0.75, label="Refined FE model")    
    plt.plot(outputs['time'], -outputs['rel_disp_roof']/H*100, linewidth=0.5, label="NMFS")
    # plt.plot(time_FE, roof_disp_FE2, linewidth=0.4, label="FEM no P-delta")
    plt.xlabel('Time (sec)')
    plt.ylabel('Roof Drift (%)')
    plt.title('Roof Displacement')
    plt.legend()
    plt.grid()
    path = r'C:\Users\geral\Documents\Research\_doc\figures\RoofDrift'
    plt.savefig(path, dpi =300)
    plt.show()

def plot_story_shear(outputs, scaling_factor, GM_number): # Plot Maximum Story Shear
    
    numStory = outputs['numStory']
    Ygrid = []
    Max_Storey_Shears = []
    for i in range(1, numStory+1):
        Ygrid.append(i-1)
        Ygrid.append(i)
        Max_Storey_Shears.append(max(abs(outputs["shear"+str(i)]))/1000) # normalized
        Max_Storey_Shears.append(max(abs(outputs["shear"+str(i)]))/1000)
        
    if scaling_factor == 0.1:
        Y_grid_FE=[]
        max_shear_FE=[]
        with open(r'C:\Users\geral\Documents\Research\Openseepy\NMFS_vs_Archetype\max_shear_FE_SC01.csv', 'r') as rf:
            reader = csv.reader(rf, delimiter=',')
            for row in reader:
                max_shear_FE.append(float(row[0]))
                Y_grid_FE.append(float(row[1])) 
        
    elif scaling_factor == 1.0:
        Y_grid_FE=[]
        max_shear_FE=[]
        # max_shear_FE2=[]
        
        with open(r'C:\Users\geral\Documents\Research\Openseepy\NMFS_vs_Archetype\max_shear_FE.csv', 'r') as rf:
            reader = csv.reader(rf, delimiter=',')
            for row in reader:
                # max_shear_FE.append(float(row[0]))
                Y_grid_FE.append(float(row[1]))
                # max_shear_FE2.append(float(row[2]))
                
        # with open(r'C:\Users\geral\Documents\Research\Openseepy\NMFS_vs_Archetype\max_shear_FE.csv', 'r') as rf:
        with open(r'C:\Users\geral\Documents\Research\Openseepy\Nasser_Marafi\SRCSWArchetypes-master\Outputs_ger\GM_'+str(GM_number)+'_max_shear.csv', 'r') as rf:
            reader = csv.reader(rf, delimiter=',')
            next(reader)
            for row in reader:
                max_shear_FE.append(float(row[0]))
    
    plt.plot(max_shear_FE, Y_grid_FE, linewidth=1.5, label="Refined FE model")
    plt.plot(Max_Storey_Shears, Ygrid, linewidth=1.5, label="NMFS")
    # plt.plot(max_shear_FE2, Y_grid_FE, linewidth=1.0, label="FEM no P-delta")
    plt.xlabel('Max. Story Shear (kN)')
    plt.ylabel('Story')
    plt.title('Story Shears')
    plt.legend()
    plt.grid()
    path = r'C:\Users\geral\Documents\Research\_doc\figures\StoryShear'
    plt.savefig(path, dpi =300)
    plt.show()
    
def plot_hysteresis(outputs, scaling_factor,GM_number): # Plot First Story Drift vs Base Shear
    
    height = outputs['height']
    
    if scaling_factor == 0.1:
        disp_1story_FE=[]
        base_shear_FE=[]
        with open(r'C:\Users\geral\Documents\Research\Openseepy\NMFS_vs_Archetype\hysteresis_FE_SC01.csv', 'r') as rf:
            reader = csv.reader(rf, delimiter=',')
            for row in reader:
                disp_1story_FE.append(float(row[0]))
                base_shear_FE.append(float(row[1])) 
        
    elif scaling_factor == 1.0:
        disp_1story_FE=[]
        base_shear_FE=[]
        # base_moment_FE=[]
        # disp_1story_FE2=[]
        # base_shear_FE2=[]
        # base_moment_FE2=[]
        # with open(r'C:\Users\geral\Documents\Research\Openseepy\NMFS_vs_Archetype\hysteresis_FE.csv', 'r') as rf:
        with open(r'C:\Users\geral\Documents\Research\Openseepy\Nasser_Marafi\SRCSWArchetypes-master\Outputs_ger\GM_'+str(GM_number)+'_drift_1st.csv', 'r') as rf:
            reader = csv.reader(rf, delimiter=',')
            for row in reader:
                disp_1story_FE.append(float(row[0]))
                # base_shear_FE.append(float(row[1]))
                # base_moment_FE.append(float(row[2]))
                # disp_1story_FE2.append(float(row[3]))
                # base_shear_FE2.append(float(row[4]))
                # base_moment_FE2.append(float(row[5]))
        with open(r'C:\Users\geral\Documents\Research\Openseepy\Nasser_Marafi\SRCSWArchetypes-master\Outputs_ger\GM_'+str(GM_number)+'_shear_1st.csv', 'r') as rf:
            reader = csv.reader(rf, delimiter=',')
            for row in reader:
                base_shear_FE.append(float(row[0]))
                
    plt.plot(disp_1story_FE, base_shear_FE, linewidth=0.5, label="Refined FE model")
    plt.plot(-outputs['rel_disp_1story']/height*100, -outputs['base_shear']/1000, linewidth=0.5, label="NMFS")
    # plt.plot(disp_1story_FE2, base_shear_FE2, linewidth=0.25, label="FEM no P-delta")
    plt.xlabel('1st InterStory Drift (%)')
    plt.ylabel('Shear Force (kN)')
    plt.title('Hysteresis Response')
    plt.legend()
    plt.grid()
    path = r'C:\Users\geral\Documents\Research\_doc\figures\HysteresisShear'
    plt.savefig(path, dpi =300)
    plt.show()
    
    # if scaling_factor == 1.0:
    #     plt.plot(disp_1story_FE, base_moment_FE, linewidth=0.5, label="Refined FE model")
    #     plt.plot(-outputs['rel_disp_1story']/height*100, outputs['moment1']/1000, linewidth=0.5, label="NMFS")
    #     # plt.plot(disp_1story_FE2, base_moment_FE2, linewidth=0.25, label="FEM no P-delta")
    #     plt.xlabel('1st InterStory Drift (%)')
    #     plt.ylabel('Base Moment (kN-m)')
    #     plt.title('Hysteresis Response')
    #     plt.legend()
    #     plt.grid()
    #     path = r'C:\Users\geral\Documents\Research\_doc\figures\HysteresisMoment'
    #     plt.savefig(path, dpi =300)
    #     plt.show()    
    
    
def plot_story_disp(outputs, scaling_factor, GM_number): # Plot Maximum Story Displacement
    
    numStory = outputs['numStory']
    height = outputs['height']
    Ygrid = []
    Max_Story_Disp = []
    for i in range(1, numStory+2):
        Ygrid.append(i-1)
        Story_Disp = outputs["disp"+str(i-1)][:]
        Max_Story_Disp.append(max(abs(Story_Disp))) # normalized
    
    if scaling_factor == 0.1:
        max_disp_FE=[]
        with open(r'C:\Users\geral\Documents\Research\Openseepy\NMFS_vs_Archetype\disp_FE_SC01.csv', 'r') as rf:
            reader = csv.reader(rf, delimiter=',')
            for row in reader:
                max_disp_FE.append(float(row[0]))

    elif scaling_factor == 1.0:
        max_disp_FE=[]
        # max_disp_FE2=[]
        # with open(r'C:\Users\geral\Documents\Research\Openseepy\NMFS_vs_Archetype\disp_FE.csv', 'r') as rf:
        with open(r'C:\Users\geral\Documents\Research\Openseepy\Nasser_Marafi\SRCSWArchetypes-master\Outputs_ger\GM_'+str(GM_number)+'_max_disp.csv', 'r') as rf:
            reader = csv.reader(rf, delimiter=',')
            for row in reader:
                max_disp_FE.append(float(row[0]))
                # max_disp_FE2.append(float(row[1]))
     
    plt.plot(max_disp_FE, Ygrid, linewidth=1.5, label="Refined FE model")
    plt.plot(Max_Story_Disp, Ygrid, linewidth=1.5, label="NMFS")
    # plt.plot(max_disp_FE2, Ygrid, linewidth=1.0, label="FEM no P-delta")
    plt.xlabel('Max. Story Displacement (m)')
    plt.ylabel('Story')
    plt.title('Story Displacements')
    plt.legend()
    plt.grid()
    path = r'C:\Users\geral\Documents\Research\_doc\figures\StoryDisp'
    plt.savefig(path, dpi =300)
    plt.show()
    
def plot_interstory_drift(outputs, scaling_factor, GM_number): # Plot Maximum Interstory Drift
    
    numStory = outputs['numStory']
    height = outputs['height'] -0.2
    Ygrid = []
    Max_Interstory_Drifts = []
    for i in range(1, numStory+1):
        Ygrid.append(i)
        Interstory_Drift = (outputs["disp"+str(i)][:] - outputs["disp"+str(i-1)][:]) / height*100
        Max_Interstory_Drifts.append(max(abs(Interstory_Drift))) 

    if scaling_factor == 0.1:
        max_drif_FE=[]
        with open(r'C:\Users\geral\Documents\Research\Openseepy\NMFS_vs_Archetype\drift_FE_SC01.csv', 'r') as rf:
            reader = csv.reader(rf, delimiter=',')
            for row in reader:
                max_drif_FE.append(float(row[0]))
        
    elif scaling_factor == 1.0:
        max_drif_FE=[]
        # max_drif_FE2=[]
        # with open(r'C:\Users\geral\Documents\Research\Openseepy\NMFS_vs_Archetype\drift_FE.csv', 'r') as rf:
        with open(r'C:\Users\geral\Documents\Research\Openseepy\Nasser_Marafi\SRCSWArchetypes-master\Outputs_ger\GM_'+str(GM_number)+'_max_drift.csv', 'r') as rf:
            reader = csv.reader(rf, delimiter=',')
            next(reader)
            for row in reader:
                max_drif_FE.append(float(row[0]))
                # max_drif_FE2.append(float(row[1]))

    plt.plot(max_drif_FE, Ygrid, linewidth=1.5, label="Refined FE model")
    plt.plot(Max_Interstory_Drifts, Ygrid, linewidth=1.5, label="NMFS")
    # plt.plot(max_drif_FE2, Ygrid, linewidth=1.0, label="FEM no P-delta")
    plt.xlabel('Max. Interstory Drift (%)')
    plt.ylabel('Story')
    plt.title('Story Drifts')
    plt.legend()
    plt.grid()
    path = r'C:\Users\geral\Documents\Research\_doc\figures\InterStoryDrift'
    plt.savefig(path, dpi =300)
    plt.show()


def plot_interstory_drift_all(Outputs_list, GM_first): # Plot Maximum Interstory Drift

    NumSimulations = int(len(Outputs_list))
    numStory = Outputs_list[0]['numStory']
    height = Outputs_list[0]['height']
    NS_Drifts = []
    NS_Drifts = []
    
    titles_font = {'fontname':'Arial'}
    
    # Get max TH storey displacements
    for GM_number in range(NumSimulations):
        outputs = Outputs_list[(GM_number)]           # N-S direction
        Ygrid = []
        Max_Interstory_Drifts = []
        for i in range(1, numStory+1):
            Ygrid.append(i)
            Interstory_Drift = (outputs["disp"+str(i)][:] - outputs["disp"+str(i-1)][:]) / height
            Max_Interstory_Drifts.append(max(abs(Interstory_Drift)))
        NS_Drifts.append(Max_Interstory_Drifts)

    # Plot unique direction  
    
    mean_values = np.zeros(numStory)
    mean_values_FE = np.zeros(numStory)
    
    for GM_number in range(NumSimulations):
        Max_Interstory_Drifts = NS_Drifts[GM_number]
        mean_values = np.add(mean_values, Max_Interstory_Drifts)
        plt.plot([i * 100 for i in Max_Interstory_Drifts], Ygrid, color = '0.6', linewidth=1.5, alpha=0.2) # label="GM"+str(GM_number+1)
    
        max_drif_FE=[]
        with open(r'C:\Users\geral\Documents\Research\Openseepy\Nasser_Marafi\SRCSWArchetypes-master\Outputs_ger\GM_'+str(GM_first+GM_number)+'_max_drift.csv', 'r') as rf:
            reader = csv.reader(rf, delimiter=',')
            next(reader)
            for row in reader:
                max_drif_FE.append(float(row[0]))
        plt.plot(max_drif_FE, Ygrid, color = 'tab:blue', linewidth=1.5, linestyle='-', alpha=0.2)        
        mean_values_FE = np.add(mean_values_FE, max_drif_FE)
    
    mean_values = mean_values/NumSimulations
    mean_values_FE = mean_values_FE/NumSimulations    
    
    plt.plot([i * 100 for i in mean_values], Ygrid, color = '0', linewidth=2.0, label="NMFS Mean")
    plt.plot(mean_values_FE, Ygrid, color = 'tab:blue', linewidth=2.0, linestyle='-', label="FE model Mean")     
    
    
    plt.xlabel('Max. Story Drift, %', fontsize=10, **titles_font)
    plt.ylabel('Stories', fontsize=10, **titles_font)
    plt.title('Story Drifts', fontsize=10, **titles_font)
    leg = plt.legend(fontsize=8, loc="lower right")
    leg.get_frame().set_edgecolor('0.0')
    leg.get_frame().set_linewidth(0.75)
    
    # plt.grid()
    plt.xlim([0,5])
    plt.ylim([1, 12])
    plt.xticks([0,1,2,3,4,5], fontsize=8)
    plt.yticks([2,4,6,8,10,12], fontsize=8)
    # plt.tight_layout()
    path = r'C:\Users\geral\Documents\Research\Openseepy\NMFS_vs_Archetype\Inter-Story_Drift'
    plt.savefig(path, bbox_inches="tight", dpi =300)
    plt.show()
   
    
def plot_floor_acceleration(outputs, scaling_factor, GM_number): # Plot Peak Floor Acceleration
    
    numStory = outputs['numStory']
    Ygrid = []
    Peak_Floor_Accel = []
    for i in range(1, numStory+2):
        Ygrid.append(i-1)
        Floor_Acc = outputs["abs_accel"+str(i-1)][:]
        Peak_Floor_Accel.append(max(abs(Floor_Acc))) # normalized
    
    if scaling_factor == 0.1:
        floor_disp_FE=[]
        # with open(r'C:\Users\geral\Documents\Research\Openseepy\NMFS_vs_Archetype\disp_FE_SC01.csv', 'r') as rf:
        #     reader = csv.reader(rf, delimiter=',')
        #     for row in reader:
        #         max_disp_FE.append(float(row[0]))

    elif scaling_factor == 1.0:
        max_acc_FE=[]
        # max_disp_FE2=[]
        # with open(r'C:\Users\geral\Documents\Research\Openseepy\NMFS_vs_Archetype\disp_FE.csv', 'r') as rf:
        with open(r'C:\Users\geral\Documents\Research\Openseepy\Nasser_Marafi\SRCSWArchetypes-master\Outputs_ger\GM_'+str(GM_number)+'_max_acc.csv', 'r') as rf:
            reader = csv.reader(rf, delimiter=',')
            for row in reader:
                max_acc_FE.append(float(row[0]))
                # max_disp_FE2.append(float(row[1]))
    max_acc_FE[0] = Peak_Floor_Accel[0]
    plt.plot(max_acc_FE, Ygrid, linewidth=1.5, label="Refined FE model")
    plt.plot(Peak_Floor_Accel, Ygrid, linewidth=1.5, label="NMFS")
    # plt.plot(max_disp_FE2, Ygrid, linewidth=1.0, label="FEM no P-delta")
    plt.xlabel('Acceleration (g)')
    plt.ylabel('Story')
    plt.title('Peak Floor Accelerations')
    plt.legend()
    plt.grid()
    path = r'C:\Users\geral\Documents\Research\_doc\figures\StoryAcc'
    plt.savefig(path, dpi =300)
    plt.show()


def plot_floor_acceleration_all(Outputs_list, GM_first): # Plot Maximum Story Displacement
    
    NumSimulations = int(len(Outputs_list))
    numStory = Outputs_list[0]['numStory']
    NS_Accel = []
    titles_font = {'fontname':'Arial'}
    
    # Get max TH storey accelerations
    for GM_number in range(NumSimulations):
        outputs = Outputs_list[(GM_number)]           # N-S direction
        Ygrid = []
        Max_Story_Accel = []
        for i in range(1, numStory+2):
            Ygrid.append(i-1)
            Story_Accel = outputs["abs_accel"+str(i-1)][:]
            Max_Story_Accel.append(max(abs(Story_Accel))) # normalized
        NS_Accel.append(Max_Story_Accel)


    # Plot unique direction  
    mean_values = np.zeros(numStory+1)
    mean_values_FE = np.zeros(numStory+1)
    
    for GM_number in range(NumSimulations):
        Max_Story_Accel = NS_Accel[GM_number]
        mean_values = np.add(mean_values, Max_Story_Accel)
        plt.plot(Max_Story_Accel, Ygrid, color = '0.6', linewidth=1.5, alpha=0.2) # label="GM"+str(GM_number+1)

        max_acc_FE=[]
        with open(r'C:\Users\geral\Documents\Research\Openseepy\Nasser_Marafi\SRCSWArchetypes-master\Outputs_ger\GM_'+str(GM_number+GM_first)+'_max_acc.csv', 'r') as rf:
            reader = csv.reader(rf, delimiter=',')
            for row in reader:
                max_acc_FE.append(float(row[0]))
        max_acc_FE[0] = Max_Story_Accel[0]
        mean_values_FE = np.add(mean_values_FE, max_acc_FE)
        plt.plot(max_acc_FE, Ygrid, color = 'tab:blue', linewidth=1.5, linestyle='--', alpha=0.2)
   
    mean_values = mean_values/NumSimulations
    mean_values_FE = mean_values_FE/NumSimulations
    
    plt.plot(mean_values, Ygrid, color = '0', linewidth=2.0, label="NMFS Mean")
    plt.plot(mean_values_FE, Ygrid, color = 'tab:blue', linewidth=2.0, linestyle='--', label="FE model Mean") 
    plt.xlabel('Acceleration, g', fontsize=10, **titles_font)
    plt.ylabel('Stories', fontsize=10, **titles_font)
    plt.title('Peak Floor Accelerations', fontsize=10, **titles_font)
    leg = plt.legend(fontsize=8, loc="lower right")
    leg.get_frame().set_edgecolor('0.0')
    leg.get_frame().set_linewidth(0.75)
    # plt.grid()
    plt.xlim([0, 1.0])
    plt.ylim([1, 12])
    plt.xticks([0,0.2,0.4,0.6,0.8,1.0], fontsize=8)
    plt.yticks([2,4,6,8,10,12], fontsize=8)
    path = r'C:\Users\geral\Documents\Research\Openseepy\NMFS_vs_Archetype\Story_Accel'
    plt.savefig(path, bbox_inches="tight", dpi =300)
    plt.show()
        


def plot_spring_response(outputs, mat_prop, backbone_curve): # Plot First Story Drift vs Base Shear
    
    numStory = outputs['numStory']
    h = outputs['height']
        
    for i in range(1, numStory+1):  # Shear Spring
        plt.plot(outputs["rel_dif_disp"+str(i)], outputs["shear"+str(i)]/1000, linewidth=0.5, label="hysteresis")
        
        if backbone_curve == 1:
            bc = mat_prop["Shear_element"+str(i)]
            plt.plot([ 0,bc[1],bc[3],bc[5] ], [ 0,bc[0]/1000,bc[2]/1000,bc[4]/1000 ], linewidth=1., label="backbone curve")           
            plt.legend()
            
        plt.xlim([-0.15, 0.15])
        # plt.ylim([-10*1000, 10*1000])
        plt.xlabel('Displacement (m)')
        plt.ylabel('Shear Force (kN)')
        plt.title('Shear Spring Response for Level '+str(i))
        plt.grid()
        plt.show()
    
    for i in range(1, numStory+1):  # Flexural Spring
        plt.plot(outputs["rel_dif_disp"+str(i)], -outputs["moment"+str(i)]/1000, linewidth=0.5, label="hysteresis")
        
        if backbone_curve == 1:
            bc = mat_prop["Flexural_element"+str(i)]
            plt.plot([ 0,bc[1]*h,bc[3]*h,bc[5]*h ], [ 0,bc[0]/1000,bc[2]/1000,bc[4]/1000 ], linewidth=1., label="backbone curve")  
            plt.legend()
            
        plt.xlim([-0.15, 0.15])
        # plt.ylim([-120*1000, 120*1000])
        plt.xlabel('Displacement (m)')
        plt.ylabel('Moment (kN-m)')
        plt.title('Flexural Spring Response for Level '+str(i))
        plt.grid()
        plt.show()
       
 

# =============================================================================
# Pushover Analysis
# =============================================================================
    
def get_pushover(numStory, height, Umax, Nsteps, nodeControl, tol, modalMatrix, pDelta):
    
    H = numStory*height
    # Now switch to pushover analysis
    op.remove('loadPattern',1) # Remove load pattern
    op.remove('loadPattern',2) # Remove load pattern
    op.reset() # Reset model to its initial state
    
    if pDelta == 1:
        
        # define GRAVITY ANALYSIS -------------------------------------------------------------
        op.timeSeries('Linear', 3)
        op.pattern('Plain', 3, 3,)
        for i in range(2, numStory+2):
            op.load(i, 0., -3448529.8709264626*0.367, 0.) # wall
            # op.load(i*100, 0., -3448529.8709264626*0.633, 0.) # leaning column
            
        op.constraints('Plain')  				# how it handles boundary conditions
        op.numberer('Plain')			    # renumber dof's to minimize band-width (optimization), if you want to
        op.system('BandGeneral')		    # how to store and solve the system of equations in the analysis
        op.algorithm('Newton')                 # use Linear algorithm for linear analysis
        op.integrator('LoadControl', 0.1)			# determine the next time step for an analysis, # apply gravity in 10 steps
        op.analysis('Static')					    # define type of analysis static or transient
        op.analyze(10)					        # perform gravity analysis
        op.loadConst('-time', 0.0)				# hold gravity constant and restart time    
        
        op.wipeAnalysis()
    
    
    # Define the nonlinear static analysis
    load_tag_static = 4
    pattern_tag_static = 4
    
    op.timeSeries('Linear', load_tag_static)
    op.pattern('Plain', pattern_tag_static, load_tag_static)
    
    # Equal reference load on each floor (uniform vertical distribution of lateral loads)
    for i in range(1, numStory+1):
        # op.load(i+1,1,0.,0.)
        op.load(i+1,(modalMatrix[i,0])*1,0.,0.)
        

    dU = Umax/Nsteps
    base_node = 1
    nodeControl = numStory+1
     
    
    op.wipeAnalysis()
    op.constraints('Plain')
    op.numberer('Plain')
    op.system('BandGeneral')
    op.test('NormDispIncr', tol, 100)
    op.algorithm('Newton')
    
    op.integrator('DisplacementControl',numStory+1, 1, dU) # Control roof displacement
    op.analysis('Static')
    
    Uplot = op.nodeDisp(numStory+1,1)
    Pplot = op.getLoadFactor(pattern_tag_static)
    Mplot = op.getLoadFactor(pattern_tag_static)
    Step = []
    for i in range(Nsteps):
        ok = op.analyze(1)
        if ok < 0:
            break
        # Pplot = np.append(Pplot,op.getLoadFactor(pattern_tag_static))
        op.reactions()
        Step = np.append(Step,i)
        Pplot = np.append(Pplot,-op.nodeReaction(base_node, 1))
        Mplot = np.append(Mplot,-op.nodeReaction(base_node, 3))
        Uplot = np.append(Uplot,op.nodeDisp(nodeControl, 1))
        
    sf = 1/1000 # kN - 0.00022480894387096/1000 for kips (e3)
    # plt.plot(Uplot/H*100,Pplot*sf) 
    # plt.xlabel('Roof Displacement (%)')
    # plt.ylabel('Base Shear, kN')
    # plt.title('Pushover Analysis')
    # plt.grid()
    # plt.show()
    
    pushoverX_FE=[]
    pushoverY_FE=[]
    pushoverY_FE2=[]
    with open(r'C:\Users\geral\Documents\Research\Openseepy\NMFS_vs_Archetype\pushover_FE.csv', 'r') as rf:
        reader = csv.reader(rf, delimiter=',')
        for row in reader:
            pushoverX_FE.append(float(row[0]))
            pushoverY_FE.append(float(row[1]))
            pushoverY_FE2.append(float(row[2]))

    
    plt.plot(Uplot/H*100, Pplot*sf, linewidth=1.5, label="NMFS")
    plt.plot(pushoverX_FE, pushoverY_FE, linewidth=1.5, label="Refined FE model")
    # plt.plot(pushoverX_FE, pushoverY_FE2, linewidth=1.0, label="FEM no P-delta")
    plt.xlabel('Roof Displacement (%)')
    plt.ylabel('Base Shear, kN')
    plt.title('Pushover Analysis')
    plt.ylim([0, 5000])
    plt.xlim([0, 2.5])
    plt.legend()
    plt.grid()
    path = r'C:\Users\geral\Documents\Research\_doc\figures\PushoverShear'
    plt.savefig(path, dpi =300)
    plt.show()
    
    pushoverMX_FE=[]
    pushoverMY_FE=[]
    pushoverMY_FE2=[]
    
    with open(r'C:\Users\geral\Documents\Research\Openseepy\NMFS_vs_Archetype\pushoverM_FE.csv', 'r') as rf:
        reader = csv.reader(rf, delimiter=',')
        for row in reader:
            pushoverMX_FE.append(float(row[0]))
            pushoverMY_FE.append(float(row[1]))
            pushoverMY_FE2.append(float(row[2]))
            
    plt.plot(Uplot/H*100, -Mplot*sf, linewidth=1.5, label="NMFS")
    plt.plot(pushoverMX_FE, pushoverMY_FE, linewidth=1.5, label="Refined FE model")
    # plt.plot(pushoverMX_FE, pushoverMY_FE2, linewidth=1.0, label="FEM no P-delta")
    plt.xlabel('Roof Displacement (%)')
    plt.ylabel('Base Moment, kN-m')
    plt.title('Pushover Analysis')
    plt.ylim([0, 150000])
    plt.xlim([0, 2.5])
    plt.legend()
    plt.grid()
    path = r'C:\Users\geral\Documents\Research\_doc\figures\PushoverMoment'
    plt.savefig(path, dpi =300)
    plt.show()
    
    
    # outputs = {"step": [Step],"force": [Pplot],"rel_disp": [Uplot]} 
 
    # with open('Pushover.csv', 'w') as f:
    #     w = csv.DictWriter(f, outputs.keys())
    #     w.writeheader()
    #     w.writerow(outputs)

    return Step, Pplot, Uplot
    