# Create NMFS structural analysis model

# =============================================================================
# =============================================================================
# NMFS MODEL - RUN MULTIPLE NON-LINEAR TIME HISTORY ANALYSES
# By: Gerald Palomino (08/08/2022)
# =============================================================================
# =============================================================================

# Implementation from the models in the following paper:
# [1] G.Palomino, K.Blowes, and C. Molina Hutt, "Evaluating post-earthquake 
# functionality and surge capacity of hospital emergency departments using
# discrete event simulation," Earthquake Spectra, vol. In review, 2022.

# Functions file "NMFS_VGH_model_functionsXY.py" file must be located in the
# current directory, and lines 18-23 of functions file must be updated.
# Define the number of Ground Motions to run in lines 180, 181

# =============================================================================
# Define path & Import modules
# =============================================================================

#path
path = r'C:\Users\geral\Documents\Research\_doc\_Electronic Supplement\S1_NMFS_Model_(simplified_structural_model)'

# inputs
GM_file = path+'\M9(49.25900,-123.12195).csv' # time_history_accelerograms

#outputs
plot_results_path = path+'\outputs_multiple_GMs'
EDPs_results_path = path+'\outputs_EDPs'

# modules
import numpy as np
import matplotlib.pyplot as plt
import openseespy.opensees as op
import csv
from scipy.optimize import fsolve
from NMFS_VGH_model_functionsXY import (get_initial_elastic_parameters, get_yield_ratio,
                                        calibrate_NMFS_model, read_ground_motion,
                                        get_modal_analysis_3d, get_th_analysis_3d,
                                        plot_shear_backbone_curves, get_pushover, 
                                        plot_flexural_backbone_curves, plot_story_accel_all,
                                        plot_displacements, plot_roof_drift,
                                        plot_story_shear, plot_hysteresis, plot_story_accel,
                                        plot_story_disp, plot_interstory_drift,
                                        plot_spring_response, plot_story_shear_all,
                                        plot_story_disp_all, plot_interstory_drift_all,
                                        get_modal_analysis
                                        )

import openseespy.postprocessing.Get_Rendering as opplot
import json
import sys
import os
import shutil



# =============================================================================
# Units (N, m)
# =============================================================================

m = 1
kg = 1
N = 1
Pa = 1
g = 9.81
inches = 0.0254*m
ft = 12*inches
kN = 1000*N
kip = 4.45*10**3*N
Ksi = 6.89*10**6*Pa
seg = 1
psf = 4.8824*kg/m/m
kPa = 1000*Pa


# =============================================================================
# Building Information - VGH Hospital
# =============================================================================

# Geometry
numStory = 4
height = 5.95*m # height per story
FloorArea = 85.1*76.8*m**2
H = numStory*height

# Materials
fpc = 28*10**6*Pa # Units: Pa

# Loads & Mass
DL_interstitial = 1.6*kPa # BBP info (163kg/m2)
LL_interstitial = 1.2*kPa # 1978 drawing (122kg/m2)
DL = 7.16*kPa # hand calculation
LL = 4.8*kPa # 1978 drawing
w_ratio = 1.1*(DL+DL_interstitial)+0.5*(LL+LL_interstitial) # add 10% wall/column mass
m_ratio = w_ratio/g
mass = (FloorArea * m_ratio) # Mass per floor


# =============================================================================
# East-West - Hazus taxonomy: C1M-LC (concrete frame building, 1977 NBCC)
# =============================================================================

# Fundamental Period
T1 = 0.671
T2 = 0.160
# T2 = 0.27*T1

# HAZUS parameters for C1M-LC (Table 5-5, 5-7, 5-14)
α1 = 0.80
α2 = 0.75
OmegaY = 1.25           # factor relating “true” yield strength to design strength
OmegaP = 1.3        # factor relating ultimate strength to yield strength (3.0 or 1.5*1.3 = 1.95)
ductility = 3.3
drift_co = 0.0333  # inter-story drift ratio at complete damage state
 
# Yield parameters
Cs = 0.19 # design strength coefficient (fraction of building’s weight)
Vd = Cs*g*mass*numStory # in Newtons
yield_shear = OmegaY*Vd
yield_rotation = 0.69616/100 # hand calc estimation - max yield rotation capacity = 0.5*εy*(lb/hb) = 0.5*(414 MPa/210000 MPa)*(11.3/1.6) = 0.69616/100 ; for beam - moment frame


# Calibration to get material properties
SFRS='frame'
mat_type = 'Hysteretic' # 'Bilin' or 'ModIMKPinching' or 'Hysteretic'



EI, GA, α0 = get_initial_elastic_parameters(T1, T2, mass, height, numStory, plot = 0)

mat_prop, alpha, modalMatrix = calibrate_NMFS_model(numStory, height, mass, T1, T2, mat_type,yield_shear,
                                                    yield_rotation, EI, GA, OmegaP, ductility, drift_co, SFRS)


# =============================================================================
# North-South - Hazus taxonomy: C2M-LC (concrete wall building, 1977 NBCC)
# =============================================================================

# Fundamental Period
T1 = 0.390
# T2 = 0.278
# T2 = 0.135
T2 = 0.27*T1

# HAZUS parameters for C2M-LC (Table 5-5, 5-7, 5-14)
α1 = 0.75
α2 = 0.75
OmegaY = 1.25          # factor relating “true” yield strength to design strength
OmegaP = 1.30          # factor relating ultimate strength to yield strength (2.5 or 1.5*1.3 = 1.95)
ductility = 3.3
drift_co = 0.0333  # inter-story drift ratio at complete damage state

# Yield parameters
Cs = 0.19 # design strength coefficient (fraction of building’s weight)
Vd = Cs*g*mass*numStory # in Newtons
yield_shear = OmegaY*Vd
yield_rotation = 0.10599/100 # hand calc estimate - max yield rotation capacity = 2*εy/lw = 2*(414 MPa/210000 MPa)/3.72m = 0.10599/100 ; for elevator walls

# Calibration to get material properties
SFRS='wall'
mat_type = 'Hysteretic' # 'Bilin' or 'ModIMKPinching' or 'Hysteretic'
EI, GA, α0 = get_initial_elastic_parameters(T1, T2, mass, height, numStory, plot = 0)

mat_prop2, alpha2, modalMatrix2 = calibrate_NMFS_model(numStory, height, mass, T1, T2, mat_type, yield_shear,
                                                       yield_rotation, EI, GA, OmegaP, ductility, drift_co, SFRS)


# =============================================================================
# OpenSees Analysis
# =============================================================================

analysisName = 'Modal Analysis'
frequency, period, Participation = get_modal_analysis_3d(numStory, height, mass, mat_type, mat_prop, mat_prop2, plot = 0)

analysisName = 'Time History Analysis'
Outputs_list = []
GM_first = 1
GM_last = 30
NumSimulations = (GM_last - GM_first) + 1

if NumSimulations != 1:
    shutil.rmtree(plot_results_path)
    os.mkdir(plot_results_path)

GM_file = GM_file
dt = 0.02
scaling_factor = 1.0

for GM_number in range(GM_first, GM_last+1):
    
    # Read ground motion from csv to list
    time, motion, motion2 = read_ground_motion(GM_file, GM_number, scaling_factor, plot = 0)
    
    # Run TH analysis
    xi = 0.025
    outputs, outputs2 = get_th_analysis_3d(numStory, height, mass, mat_type, mat_prop, mat_prop2, time, motion, motion2, dt, xi, GM_number, NumSimulations)
    
    # Save outputs for run GM_number
    Outputs_list.append(outputs)
    Outputs_list.append(outputs2)

# =============================================================================
# Animation & Pushover Analysis
# =============================================================================

# Display the active model with node tags only
# plot.plot_model("nodes")
# plot.plot_modeshape(1, 2)

#Animation - TH analysis
# op.wipe()
# Model = 'NMFS'
# LoadCase = 'Transient'
# ani = opplot.animate_deformedshape( Model, LoadCase, 2., scale=300)
# #%matplotlib qt
# #%matplotlib inline

analysisName = 'Pushover Analysis'
Umax = 2.5/100*numStory*height
Nsteps = 500
tol = 1.*10**-8
# Step, Pplot, Uplot, Step2, Pplot2, Uplot2  = get_pushover(numStory, height, Umax, Nsteps, tol, modalMatrix, modalMatrix2)


# # =============================================================================
# # Plots - Set N (gorund motion number)
# # =============================================================================

if NumSimulations == 1 :
    outputs = Outputs_list[0]
    outputs2 = Outputs_list[1]
    
    plot_XX = 0
    plot_YY = 0
    plot_XY = 1
    direction = [plot_XX,plot_YY,plot_XY]
    
    # Backbone Shear/Flexural Spring Curve Plots
    # plot_shear_backbone_curves(numStory, mat_prop, mat_prop2)
    # plot_flexural_backbone_curves(numStory, mat_prop, mat_prop2)
    
    # plot_displacements(outputs) # Plot Time vs Displacements
    plot_roof_drift(outputs, outputs2, direction, GM_number) # Plot Time vs Roof Drift
    plot_story_shear(outputs, outputs2, direction) # Plot Maximum Story Shear
    plot_story_accel(outputs, outputs2, direction) # Plot Peak Floor Accelerations
    plot_story_disp(outputs, outputs2, direction) # Plot Maximum Story Drift
    plot_interstory_drift(outputs, outputs2, direction) # Plot Maximum Interstory Drift
    # plot_hysteresis(outputs, outputs2, direction) # Plot First Inter-Story Drift vs Base Shear
    # plot_spring_response(outputs, mat_prop, backbone_curve = 1)
    
    # WARNING MESSAGES
    Total_time = len(time)*dt
    time_max = len(Outputs_list[0]['time'])*dt
    GM_ID = GM_number
    print ('-----------------------------------------')
    print ('-----------------------------------------')
    if time_max == Total_time:
        print (str(GM_ID)+" Run Complete")
    else:
        print (str(GM_ID)+" Run Incomplete, fails at "+str(time_max)+"s")
        
# =============================================================================
# Plots all simulations
# =============================================================================

# #%matplotlib qt
# #%matplotlib inline
plt.rcParams["figure.figsize"] = (3.25,2.20)
# plt.rcParams["figure.figsize"] = plt.rcParamsDefault["figure.figsize"]
plt.rcParams['axes.linewidth'] = 1.25

if NumSimulations != 1 :
       
    # plot_displacements(outputs) # Plot Time vs Displacements
    # plot_roof_drift_all(Outputs_list) # Plot Time vs Roof Drift
    # plot_story_shear_all(Outputs_list) # Plot Maximum Story Shear
    EW_Accel, NS_Accel = plot_story_accel_all(Outputs_list) # Plot Peak Floor Accelerations
    # EW_Disps, NS_Disps = plot_story_disp_all(Outputs_list) # Plot Maximum Story Drift
    EW_Drifts, NS_Drifts = plot_interstory_drift_all(Outputs_list) # Plot Maximum Interstory Drift
    
    # plot_hysteresis(Outputs_list) # Plot First Inter-Story Drift vs Base Shear
    # plot_spring_response(outputs, mat_prop, backbone_curve = 1)    
    
    print ('-----------------------------------------')
    print ('-----------------------------------------')
    # WARNING MESSAGES
    Total_time = len(time)*dt
    for GM_number in range(NumSimulations):
        time_max = len(Outputs_list[GM_number*2]['time'])*dt
        GM_ID = GM_number + GM_first
        if time_max == Total_time:
            print (str(GM_ID)+" Run Complete")
        else:
            print (str(GM_ID)+" Run Incomplete, fails at "+str(time_max)+"s")
    
# =============================================================================
# Write Pelicun EDP file    
# =============================================================================

    # Get Residual Drifts
    EW_ResidualDrifts=[]
    NS_ResidualDrifts=[]
    for GM_number in range(NumSimulations):
        outputs = Outputs_list[2*(GM_number)]           # E-W direction
        outputs2 = Outputs_list[2*(GM_number) + 1]      # N-S direction
        EW_ResidualDrifts.append(outputs["residual_drift"])
        NS_ResidualDrifts.append(outputs2["residual_drift"])

    output_file_path = EDPs_results_path+'\EDP_VGH_M9.csv'
    header = ['']
    for i in range(numStory):
        header.append('1-PID-'+str(i+1)+'-1')
        header.append('1-PID-'+str(i+1)+'-2')
    for i in range(numStory+1):
        header.append('1-PFA-'+str(i)+'-1')
        header.append('1-PFA-'+str(i)+'-2')  
    header.append('1-RID-1-1')
    header.append('1-RID-1-2')
    
    with open(output_file_path, "w", newline='') as fp:
        wr = csv.writer(fp, dialect='excel')
        wr.writerow(header)
        for k in range(NumSimulations):
            drifts = EW_Drifts[k]
            drifts2 = NS_Drifts[k]
            accel = EW_Accel[k]
            accel2 = NS_Accel[k]    
            row_k = [k]
            for i in range(numStory):
                row_k.append(drifts[i])
                row_k.append(drifts2[i])
            for i in range(numStory+1):
                row_k.append(accel[i]*32.1740)  # g = 32.1740 ft/s2
                row_k.append(accel2[i]*32.1740) # g = 32.1740 ft/s2
            row_k.append(EW_ResidualDrifts[k])
            row_k.append(NS_ResidualDrifts[k])
            wr.writerow(row_k)
            # #Additional files for Pelicun
            Output_for_Pelicun = EDPs_results_path + '\EDP_VGH_M9_GM_'+str(k+1)+'.csv'
            row_k[0]=0
            with open(Output_for_Pelicun, "w", newline='') as fp2:
                wr2 = csv.writer(fp2, dialect='excel')
                wr2.writerow(header)
                wr2.writerow(row_k)
            fp2.close()        
    fp.close()
    
    
    
    
    