# =============================================================================
# =============================================================================
# FUNCTIONS FILE
# NMFS MODEL - RUN MULTIPLE NON-LINEAR TIME HISTORY ANALYSES
# By: Gerald Palomino (08/08/2022)
# =============================================================================
# =============================================================================

# Implementation from the models in the following paper:
# [1] G.Palomino, K.Blowes, and C. Molina Hutt, "Evaluating post-earthquake 
# functionality and surge capacity of hospital emergency departments using
# discrete event simulation," Earthquake Spectra, vol. In review, 2022.

# =============================================================================
# Define path & Import modules
# =============================================================================

#path
path = r'C:\Users\geral\Documents\Research\_doc\_Electronic Supplement\S1_NMFS_Model_(simplified_structural_model)'

#outputs
plot_results_path = path+'\outputs_multiple_GMs'
plot_GM_file = path+'\outputs_GM_plots' # accelerograms plots

# modules
import numpy as np
import matplotlib.pyplot as plt
import openseespy.opensees as op
import csv
from scipy.optimize import fsolve
import scipy.linalg as la
import openseespy.postprocessing.Get_Rendering as opplot

import shutil

# =============================================================================
# Initial Functions
# =============================================================================
  
def get_modal_analysis(numStory, height, mass, mat_type, mat_prop, plot):

    # remove existing model
    op.wipe()
    
    # set modelbuilder
    op.model('basic', '-ndm', 2, '-ndf', 3)
    
    # Nodes and boundary conditions
    op.node(1,0.,0.); op.fix(1, 1, 1, 1)
    
    for i in range(2, numStory+2):
        op.node(i,0.,height*(i-1))
        op.fix(i, 0., 0., 0.) # no fixed
            
    for i in range(2, numStory+2):
        op.mass(i, mass, 0, 0.) 
    
         
    # Define material        
    for i in range(1, numStory+1):
        # Shear springs material
        shear_mat_tag = i
        mat_property1 = mat_prop["Shear_element"+str(i)]
        op.uniaxialMaterial(mat_type, shear_mat_tag, *mat_property1)
        # Flexural spring material
        flex_mat_tag = (i+numStory)
        mat_property2 = mat_prop["Flexural_element"+str(i)]
        op.uniaxialMaterial(mat_type, flex_mat_tag, *mat_property2)
    
    op.uniaxialMaterial('Elastic', 1+2*numStory, 1.*10**10)
    
    # op.geomTransf('Linear', 1, '-jntOffset', 0,0, 0,0)
    # Define elements
    for i in range(1, numStory+1):
        # Shear springs
        eleTag = i
        bot_node = i
        top_node = i+1
        shear_mat_tag = i
        flex_mat_tag = (i+numStory)
        op.element('twoNodeLink', eleTag, bot_node, top_node, '-mat', 1+2*numStory, shear_mat_tag, flex_mat_tag, "-dir", 1,2,3, '-pDelta', 0.5,0.5 , '-shearDist', 0.5,0.5, '-doRayleigh', 1)  
        # op.element('twoNodeLink', eleTag, bot_node, top_node, '-mat', 1+2*numStory, shear_mat_tag, flex_mat_tag, "-dir", 1,2,3,'-shearDist', 0.5,0.5, '-doRayleigh', 1)  
        #op.element('twoNodeLink', eleTag, bot_node, top_node, '-mat', mat_tag, "-dir", 2, 3, '-doRayleigh', 1)
 
    # OpenSees eigenvalue analysis
    omegaSquared = op.eigen('-fullGenLapack', numStory) 
    
    # Periods and frequencies of the structure
    omega = [i**0.5 for i in omegaSquared]
    frequency = [i/(2*np.pi) for i in omega]
    period = [2*np.pi/i for i in omega]
    
    # Mode Shapes
    modalMatrix = np.zeros(shape=(numStory+1,numStory))
    for i in range(0, numStory+1):
        for j in range(0, numStory):
            modalMatrix[i,j] = op.nodeEigenvector(i+1, j+1, 1)
    h = np.zeros(shape=(numStory+1))
    for i in range(0, numStory+1):
        h[i] = i
    
    # Plot Mode shape n
    if plot == 1:
        for i in range(0, numStory):
            n = i
            plt.plot(modalMatrix[:,n], h, [0, 0], [0, numStory])
            plt.xlabel('Mode '+str(n+1))
            plt.ylabel('Story Level')
            plt.title('Normalized Modal Shape')
            plt.grid()
            plt.show()
    
# Participation Factors

    r = -1*np.ones(numStory)
    M = np.zeros((numStory, numStory))
    for i in range(numStory):
        M[i, i] = 1
    
    phiMatrix = modalMatrix[1:numStory+1,:]
    alpha = (np.dot(np.dot(phiMatrix.T, M), r))
    Participation = (alpha**2)/sum(alpha**2)
       
    return frequency, period, modalMatrix, Participation


def get_initial_elastic_parameters(T1, T2, mass, height, numStory, plot):
    
    α0 = 1.0; error = 1.0 # Initial values
    
    while error > 0.000001:
    
        α0_guess = α0
        func = lambda γn : 2 + (2 + α0_guess**4/((γn**2)*(γn**2+α0_guess**2)))*np.cos(γn)*np.cosh((γn**2+α0_guess**2)**0.5) + (α0_guess**2/(γn*(γn**2+α0_guess**2)**0.5))*np.sin(γn)*np.sinh((γn**2+α0_guess**2)**0.5)
    
        # Use the numerical solver to find the roots
        γn_initial_guess = np.array([0.5, 5.0])
        γ = fsolve(func, γn_initial_guess)
        γ1 = γ[0]
        γ2 = γ[1]

        # Solve equations to get γn as a function of α0_guess
        func = lambda α0_guess : T2/T1 - (γ1/γ2)*( ((γ1)**2+α0_guess**2)/((γ2)**2+α0_guess**2) )**0.5
    
        # Use the numerical solver to find the roots
        α0 = fsolve(func, α0_guess)
        error = abs( (α0 - α0_guess) / α0 )
    
    if plot == 1:
        # Plot it
        α0_guess = α0
        func = lambda γn : 2 + (2 + α0_guess**4/((γn**2)*(γn**2+α0_guess**2)))*np.cos(γn)*np.cosh((γn**2+α0_guess**2)**0.5) + (α0_guess**2/(γn*(γn**2+α0_guess**2)**0.5))*np.sin(γn)*np.sinh((γn**2+α0_guess**2)**0.5)
        γn1 = np.linspace(0.1, 5, 200)
        plt.plot(γn1, func(γn1))
        plt.xlabel("γn")
        plt.ylabel("expression value")
        plt.grid()
        plt.show()
            
        # Plot α0_guess
        α0_guess = np.linspace(0, 5, 201)
        plt.plot(α0_guess, func(α0_guess))
        plt.xlabel("α0")
        plt.ylabel("expression value")
        plt.grid()
        plt.show()
        
    g = 9.81    
    # Determine EI & GA
    ang_freq1 = 2*np.pi / T1
    p = mass*g/height
    H = height*numStory
    EI = ang_freq1**2 * p * H**4 /(γ1**2)/(γ1**2 + α0**2)
    GA = EI * (α0/H)**2
    
    return EI, GA, α0

def get_yield_ratio(α):
    Niterations = 1000
    K2 = []
    K3 = []
    for i in range(Niterations+1):
        eps = (i)/Niterations
        K2.append( 6/(α**3) * ( (α*np.sinh(α)+1)*(np.sinh(α*eps)/np.cosh(α)) - α*np.cosh(α*eps) + α*(1-eps) ) )
        K3.append( 2/(α**2) * ( (α*np.sinh(α)+1)*(np.cosh(α*eps)/np.cosh(α)) - α*np.sinh(α*eps) - 1 ) )
    Ky = (α**2)/3 * (max(K2))/(max(K3))
    return Ky



def calibrate_NMFS_model(numStory, height, mass, T1, T2, mat_type, yield_shear, yield_rotation,
                         EI, GA, OmegaP, ductility, drift_co, SFRS):
    
    H = numStory*height
    # =============================================================================
    # 1. Calibration of the Elastic Parameters
    # =============================================================================
    EI2=EI
    GA2=GA
    errorT2= 1.0 
    while errorT2 > 0.001:      
        errorT1= 1.0
        while errorT1 > 0.001:
            EI=EI2
            GA=GA2
        # =============================================================================
        # 2. Calibration of the Yield Parameters
        # =============================================================================
            yield_moment = yield_rotation*EI/height    
            rf = np.zeros(numStory)
            rs = np.zeros(numStory)
            for i in range(numStory):
                rf_temp = 1-i*(1-0.6)/(numStory-1) #ratio between the shear strength of the ith story and that of the ground story
                rf[i] = rf_temp
                rs_temp = 1-(i*(i+1))/(numStory*(numStory+1)) #ratio between the shear strength of the ith story and that of the ground story
                rs[i] = rs_temp
            My = yield_moment*rf   #660,000 
            # My = yield_moment*np.ones(numStory)
            Vy = yield_shear*rs
            # Vy = yield_shear*np.ones(numStory)
            Dy = Vy*(height)/(GA)
            Ry = My*(height)/(EI)
        
        # =============================================================================
        # 3. Calibration of the Peak Point
        # =============================================================================    
        
            # Get ultimate shear and moment
            Vp = OmegaP*Vy
            Mp = OmegaP*My
            method = 'Ductility_factor_method' # 'Stiffness reduction method' or 'Ductility factor method'
        
            if method == 'Stiffness_reduction_method': # Method 01: Get Stiffness reduction factor 'eta' 
                eta = 0.7 # or 0.875 from ACI 318-08 10.10.4.1 --- or 0.4
                ErI = eta*EI # reduced bending stiffness
                GrA = eta*GA # reduced shear stiffness
                Dp = Vp*height/GrA
                Rp = Mp*height/ErI   
            elif method == 'Ductility_factor_method':
                Dp = ductility*OmegaP*Dy
                Rp = ductility*OmegaP*Ry  
                
            # Get ultimate displacement and rotation
            Du = drift_co*height # Disp. at Threshold of Complete Damage State
            Ru = drift_co
        
        # =============================================================================
        # 4. Calibration of the Hysteretic Parameter
        # =============================================================================      
            
            # Hysteretic behavior - Pinching model proposed by Steelman and Hajjar (2009)
            k_degradation = 0.6 # 0.6/0.3/0.1 degradation factor for short/mod/long earthquake C1M Low Code (Table 5.18)
            # tau = Ap/Ab
            tau = k_degradation
        
        # =============================================================================
        # CREATE MATERIAL
        # =============================================================================    
    
            mat_prop = {}
            if SFRS == 'frame':
                pinchX = 0.4 # to calibrate
                pinchY = 0.7 # to calibrate
                damage1 = 0 # ductility-1
                damage2 = 0 # Eii/Eult
                beta = 0.0 # E-W direction - concrete moment frame direction
            if SFRS == 'wall':
                pinchX = 0.4 # to calibrate 0.4
                pinchY = 0.7 # to calibrate 0.7 
                damage1 = 0 # ductility 0
                damage2 = 0 # Eii/Eult 0
                beta = 0.0 # N-S direction - concrete shear wall direction 0
            for i in range(1, numStory+1):
                mat_prop["Shear_element"+str(i)] = []
            for i in range(1, numStory+1):
                mat_prop["Flexural_element"+str(i)] = []
            
            for i in range(numStory):  # Shear Spring
                p1 = [Vy[i], Dy[i]]
                p2 = [Vp[i], Dp[i]]
                p3 = [Vp[i], Du]
                n1 = [-Vy[i], -Dy[i]]
                n2 = [-Vp[i], -Dp[i]]
                n3 = [-Vp[i], -Du]
                #        *p1, *p2, *p3=p2, *n1, *n2, *n3=n2, pinchX, pinchY, damage1, damage2, beta)
                mat_prop["Shear_element"+str(i+1)] = [*p1, *p2, *p3, *n1, *n2, *n3, pinchX, pinchY, damage1, damage2, beta]
    
            for i in range(numStory):  # Flexural Spring              
                p1 = [My[i], Ry[i]]
                p2 = [Mp[i], Rp[i]]
                p3 = [Mp[i], Ru]
                n1 = [-My[i], -Ry[i]]
                n2 = [-Mp[i], -Rp[i]]
                n3 = [-Mp[i], -Ru]
                #        *p1, *p2, *p3=p2, *n1, *n2, *n3=n2, pinchX, pinchY, damage1, damage2, beta)
                mat_prop["Flexural_element"+str(i+1)] = [*p1, *p2, *p3, *n1, *n2, *n3, pinchX, pinchY, damage1, damage2, beta]
                
            frequency, period, modalMatrix, Participation = get_modal_analysis(numStory, height, mass, mat_type, mat_prop, plot = 0)
            
            # update NMFS model
            # r0 = Dy[0]/height
            # yield_moment = float(r0*EI/height)
            α = float((numStory*height)*np.sqrt(GA/EI)) # participation of flexural and shear deformations
            # Ky = get_yield_ratio(α)
            # yield_moment = yield_shear * H / Ky
            
            errorT1 = T1 - period[0]
            if errorT1 > 0.0 :
                EI2 = EI*0.9
            else :
                EI2 = EI*1.1        
            errorT1 = abs (errorT1)   
        errorT2 = T2 - period[1]
        if errorT2 > 0.0 :
            GA2 = GA*0.9
        else :
            GA2 = GA*1.1
        errorT2 = abs (errorT2)
    
    return mat_prop, α, modalMatrix   


# =============================================================================
# Modal Analsys
# =============================================================================
    
def get_modal_analysis_3d(numStory, height, mass, mat_type, mat_prop, mat_prop2, plot):

    # remove existing model
    op.wipe()
    
    # set modelbuilder
    op.model('basic', '-ndm', 3, '-ndf', 6)
    
    # Nodes and boundary conditions
    op.node(1, 0., 0., 0.); op.fix(1, 1, 1, 1, 1, 1, 1)
    
    for i in range(2, numStory+2):
        op.node(i, 0., 0., height*(i-1))
        op.fix(i, 0, 0, 0, 0, 0, 1) # no fixed
            
    for i in range(2, numStory+2):
        op.mass(i, mass, mass, 0, 1.0e-4, 1.0e-4, 1.0e-4)  
    
    # Define material        
    for i in range(1, numStory+1):
        ## X-X direction
        # Shear springs material
        shear_mat_tag = i
        mat_property1 = mat_prop["Shear_element"+str(i)]
        op.uniaxialMaterial(mat_type, shear_mat_tag, *mat_property1)
        # Flexural spring material
        flex_mat_tag = (i+numStory)
        mat_property2 = mat_prop["Flexural_element"+str(i)]
        op.uniaxialMaterial(mat_type, flex_mat_tag, *mat_property2)

        ## Y-Y direction
        # Shear springs material
        shear_mat_tag2 = (i+100)
        mat_property3 = mat_prop2["Shear_element"+str(i)]
        op.uniaxialMaterial(mat_type, shear_mat_tag2, *mat_property3)
        # Flexural spring material
        flex_mat_tag2 = ((i+100)+numStory)
        mat_property4 = mat_prop2["Flexural_element"+str(i)]
        op.uniaxialMaterial(mat_type, flex_mat_tag2, *mat_property4) 
    
    op.uniaxialMaterial('Elastic', 1+2*numStory, 1.*10**10) # additional material for axial direction
    
    # op.geomTransf('Linear', 1, '-jntOffset', 0,0, 0,0)
    # Define elements
    for i in range(1, numStory+1):
        ## X-X & YY direction
        eleTag = i
        bot_node = i
        top_node = i+1
        shear_mat_tag = i
        flex_mat_tag = (i+numStory)
        shear_mat_tag2 = (i+100)
        flex_mat_tag2 = ((i+100)+numStory)
        rigid_mat_tag = (1+2*numStory)
        op.element('twoNodeLink', eleTag, bot_node, top_node, '-mat', rigid_mat_tag, shear_mat_tag, shear_mat_tag2, flex_mat_tag2, flex_mat_tag, '-dir', 1,2,3,5,6, '-orient',0,0,1,1,0,0)  
        # op.element('twoNodeLink', eleTag, bot_node, top_node, '-mat', 1+2*numStory, shear_mat_tag, flex_mat_tag, "-dir", 1,2,3,'-shearDist', 0.5,0.5, '-doRayleigh', 1)  
        #op.element('twoNodeLink', eleTag, bot_node, top_node, '-mat', mat_tag, "-dir", 2, 3, '-doRayleigh', 1)
 
    # OpenSees eigenvalue analysis
    eigenvalues = numStory*2
    omegaSquared = op.eigen('-fullGenLapack', eigenvalues) 
    # omegaSquared = op.eigen('-genBandArpack', 4) 
    
    # Periods and frequencies of the structure
    omega = [i**0.5 for i in omegaSquared]
    frequency = [i/(2*np.pi) for i in omega]
    period = [2*np.pi/i for i in omega]
    
    # Mode Shapes
    modalMatrix = np.zeros(shape=(numStory+1,eigenvalues))
    # modalMatrix_YY = np.zeros(shape=(numStory+1,numStory))
    for j in range(0, eigenvalues):
        for i in range(1, numStory+1):
            modalMatrix[i,j] = op.nodeEigenvector(i+1, j+1,1)
            # modalMatrix_YY[i,j] = op.nodeEigenvector(i+1, j+1,2)
            # if op.nodeEigenvector(i+1, j+1,2)==0:
            #     modalMatrix[i,j] = op.nodeEigenvector(i+1, j+1,1)
            # else:
            #     modalMatrix_YY[i,j] = op.nodeEigenvector(i+1, j+1,2)
            
    h = np.zeros(shape=(numStory+1))
    for i in range(0, numStory+1):
        h[i] = i
    
    # Plot Mode shape n
    if plot == 1:
        opplot.plot_model("nodes")
        for i in range(0, eigenvalues-2):
            # n = i
            # plt.plot(modalMatrix[:,n], h, [0, 0], [0, numStory])
            # plt.xlabel('Mode '+str(n+1))
            # plt.ylabel('Story Level')
            # plt.title('Normalized Modal Shape')
            # plt.grid()
            # plt.show()
            # plot.plot_model("elements")
            # plot.plot_model("nodes","elements")
            opplot.plot_modeshape(i+1,5000)

# Participation Factors

    r = -1*np.ones(numStory)
    M = np.zeros((numStory, numStory))
    for i in range(numStory):
        M[i, i] = 1
    
    phiMatrix = modalMatrix[1:numStory+1,:]
    alpha = (np.dot(np.dot(phiMatrix.T, M), r))
    Participation = (alpha**2)/sum(alpha**2)
       
    return frequency, period, Participation


# =============================================================================
# Read Ground Motion from csv to list
# =============================================================================

def read_ground_motion(GM_file, GM_number, scaling_factor, plot):

    time=[]
    motion=[]
    motion2=[]
    with open(GM_file, 'r') as rf:
        next(rf)
        reader = csv.reader(rf, delimiter=',')
        for row in reader:
            time.append(float(row[0]))
            motion.append(float(row[(GM_number-1)*3+1])*scaling_factor)
            motion2.append(float(row[(GM_number-1)*3+2])*scaling_factor)

    if plot == 1:
        plt.plot(time,motion, linewidth=0.5, label="NMFS W-E")
        plt.plot(time,motion2, linewidth=0.5, label="NMFS N-S")
        plt.xlabel('Time (sec)')
        plt.ylabel('Ground Acceleration (g)')
        plt.title('Earthquake Ground Motion - GM '+str(GM_number))
        plt.legend()
        plt.grid()
        plt.xlim([0, 500])
        plt.ylim([-0.25, 0.25])
        path = plot_GM_file + '\GM_'+str(GM_number)
        plt.savefig(path, dpi =300)
        plt.show()
    return time, motion, motion2


# =============================================================================
# Time History Analysis
# =============================================================================
    
def get_th_analysis_3d(numStory, height, mass, mat_type, mat_prop, mat_prop2, time, motion, motion2, dt, xi, GM_number, NumSimulations):

    # remove existing model
    op.wipe()
    
    # set modelbuilder
    op.model('basic', '-ndm', 3, '-ndf', 6)
    
    # Nodes and boundary conditions
    op.node(1, 0., 0., 0.); op.fix(1, 1, 1, 1, 1, 1, 1)
    # op.node(1*100,10.,0.); op.fix(1*100, 1, 1, 1) # leaning column 5m apart
    
    for i in range(2, numStory+2):
        op.node(i, 0., 0., height*(i-1))
        op.fix(i, 0, 0, 0, 0, 0, 1) # no fixed
        # # leaning column 5m apart
        # op.node(i*100,10.,height*(i-1))
        # op.fix(i*100, 0., 0., 1.)  
        
    for i in range(2, numStory+2):
        op.mass(i, mass, mass, 1.0e-4, 1.0e-4, 1.0e-4, 1.0e-4) 
        # op.mass(i*100, 0, mass, 0.) # leaning column 5m apart
        
    # Define material        
    for i in range(1, numStory+1):
        ## X-X direction
        # Shear springs material
        shear_mat_tag = i
        mat_property1 = mat_prop["Shear_element"+str(i)]
        op.uniaxialMaterial(mat_type, shear_mat_tag, *mat_property1)
        # Flexural spring material
        flex_mat_tag = (i+numStory)
        mat_property2 = mat_prop["Flexural_element"+str(i)]
        op.uniaxialMaterial(mat_type, flex_mat_tag, *mat_property2)
        
        ## Y-Y direction
        # Shear springs material
        shear_mat_tag2 = (i+100)
        mat_property3 = mat_prop2["Shear_element"+str(i)]
        op.uniaxialMaterial(mat_type, shear_mat_tag2, *mat_property3)
        # Flexural spring material
        flex_mat_tag2 = ((i+100)+numStory)
        mat_property4 = mat_prop2["Flexural_element"+str(i)]
        op.uniaxialMaterial(mat_type, flex_mat_tag2, *mat_property4)        
    
    op.uniaxialMaterial('Elastic', 1+2*numStory, 1.*10**15) # additional material for axial direction
    # op.uniaxialMaterial('Steel02',7000,379e6,202.7e9,0.01,18.5,0.925,0.15) # leaning column 5m apart
    
    # op.geomTransf('Linear', 1, '-jntOffset', 0,0, 0,0)
    # Define elements
    for i in range(1, numStory+1):
        ## X-X & YY direction
        eleTag = i
        bot_node = i
        top_node = i+1
        shear_mat_tag = i
        flex_mat_tag = (i+numStory)
        shear_mat_tag2 = (i+100)
        flex_mat_tag2 = ((i+100)+numStory)
        rigid_mat_tag = (1+2*numStory)
        op.element('twoNodeLink', eleTag, bot_node, top_node, '-mat', rigid_mat_tag, shear_mat_tag, shear_mat_tag2, flex_mat_tag2, flex_mat_tag, '-dir', 1,2,3,5,6, '-orient',0,0,1,1,0,0)  
        # op.element('twoNodeLink', eleTag, bot_node, top_node, '-mat', 1+2*numStory, shear_mat_tag, flex_mat_tag, "-dir", 1,2,3,'-shearDist', 0.5,0.5, '-doRayleigh', 1)  
        # op.element('twoNodeLink', eleTag, bot_node, top_node, '-mat', mat_tag, "-dir", 2, 3, '-doRayleigh', 1)
        
        # eleTag_lc = i*100
        # lc_mat_tag = (1+2*numStory)
        # op.element('truss', eleTag_lc, bot_node*100, top_node*100, 223.53e-4, lc_mat_tag) # leaning column 5m apart
                
        # # Rigid links
        # left_node = top_node
        # right_node = top_node*100
        # op.equalDOF(left_node, right_node, 1)
        
    # Damping ratios for modes
    zeta1 = xi
    zeta2 = xi
    
    # OpenSees eigenvalue analysis
    omegaSquared = op.eigen('-fullGenLapack', numStory) 
    omega1 = omegaSquared[0]**0.5
    omega2 = omegaSquared[2]**0.5 # be careful! - change value so you analyze correct duration.
    
    # Compute Rayleigh damping coefficients
    A = np.zeros(shape=(2,2))
    A[0,0] = 1/omega1; A[0,1] = omega1
    A[1,0] = 1/omega2; A[1,1] = omega2
    
    b = np.zeros(2)
    b[0] = 2*zeta1
    b[1] = 2*zeta2
    
    # Solve linear system Ax=b
    x = np.linalg.solve(A,b)
    
    #             M    KT  KI  Kn
    op.rayleigh(x[0],0.0,0.0,x[1])
    
    
    # Define the dynamic analysis
    load_tag_dynamic = 2 # X-X direction
    pattern_tag_dynamic = 2 # X-X direction
    load_tag_dynamic2 = 3 # X-X direction
    pattern_tag_dynamic2 = 3 # Y-Y direction
    
    values = [i * -1 for i in motion]  # should be negative ????
    values2 = [i * -1 for i in motion2]  # should be negative ????
    op.timeSeries('Path', load_tag_dynamic, '-dt', dt, '-values', *values)
    op.timeSeries('Path', load_tag_dynamic2, '-dt', dt, '-values', *values2)
    op.pattern('UniformExcitation', pattern_tag_dynamic, 1, '-accel', load_tag_dynamic, '-fact', 9.81)
    op.pattern('UniformExcitation', pattern_tag_dynamic2, 2, '-accel', load_tag_dynamic2, '-fact', 9.81)
    
    # Animation
    Model = 'NMFS'
    LoadCase = 'Transient'
    opplot.createODB(Model, LoadCase, Nmodes = 1, deltaT = 2., recorders = [])
    
    op.algorithm('Newton')
    op.system('SparseGeneral')
    op.numberer('RCM')
    op.constraints('Transformation')
    op.integrator('Newmark',0.5,0.25)
    op.analysis('Transient')
    
    # Analysis time step and number of steps
    tol = 1.0e-8
    iterations = 1000
    op.test('EnergyIncr', tol, iterations, 0, 2)
    Nsteps = len(time)
        
    base_node = 1
    firstStory_node = 2
    roof_node = numStory+1
    
    # X-X direction
    outputs = {
        "numStory": numStory,
        "height": height,
        "time": [],
        "rel_disp_1story": [],
        "rel_disp_roof": [],
        "rel_accel": [],
        "rel_vel": [],
        "base_shear": [],
        "base_axial_lc": [],
        "residual_drift": []
    }
    
    # Y-Y direction
    outputs2 = {
        "numStory": numStory,
        "height": height,
        "time": [],
        "rel_disp_1story": [],
        "rel_disp_roof": [],
        "rel_accel": [],
        "rel_vel": [],
        "base_shear": [],
        "base_axial_lc": [],
        "residual_drift": []
    }
    
    # StoryShears  = {}
    # StoryDisp = {}
    for i in range(1, numStory+1):
            # X-X direction
            outputs["shear"+str(i)] = []
            outputs["moment"+str(i)] = []
            outputs["rel_dif_disp"+str(i)] = []
            # Y-Y direction
            outputs2["shear"+str(i)] = []
            outputs2["moment"+str(i)] = []
            outputs2["rel_dif_disp"+str(i)] = []
    for i in range(1, numStory+2):
            # X-X direction
            outputs["disp"+str(i-1)] = []
            outputs["abs_accel"+str(i-1)] = []
            # Y-Y direction
            outputs2["disp"+str(i-1)] = []
            outputs2["abs_accel"+str(i-1)] = []
            
    for k in range(Nsteps):
        ok = op.analyze(1,dt)
        if ok < 0:
            break
        curr_time = op.getTime()
        # X-X direction
        outputs["time"].append(curr_time)
        outputs["rel_disp_1story"].append(op.nodeDisp(firstStory_node, 1)) # DOF=1 (X direction)
        outputs["rel_disp_roof"].append(op.nodeDisp(roof_node, 1))
        outputs["rel_accel"].append(op.nodeAccel(base_node, 1)/9.8)
        outputs["rel_vel"].append(op.nodeVel(roof_node, 1))
        # Y-Y direction
        outputs2["time"].append(curr_time)
        outputs2["rel_disp_1story"].append(op.nodeDisp(firstStory_node, 2)) # DOF=2 (Y direction)
        outputs2["rel_disp_roof"].append(op.nodeDisp(roof_node, 2))
        outputs2["rel_accel"].append(op.nodeAccel(base_node, 2)/9.8)
        outputs2["rel_vel"].append(op.nodeVel(roof_node, 2))
        
        op.reactions()
        # X-X direction
        outputs["base_shear"].append(-op.nodeReaction(base_node, 1))  # Negative since diff node
        # outputs["base_axial_lc"].append(-op.nodeReaction(base_node*100, 2))  # Reaction Leaning Column
        # Y-Y direction
        outputs2["base_shear"].append(-op.nodeReaction(base_node, 2))
        
        for i in range(1, numStory+1):
            node = i
            # X-X direction
            outputs["shear"+str(i)].append(-op.eleForce(node, 1)) # DOF=1, displacement in X
            outputs["moment"+str(i)].append(-op.eleForce(node, 5)) # DOF=5, rotation in Y axis (Xplane)
            outputs["rel_dif_disp"+str(i)].append(op.nodeDisp(node+1, 1)-op.nodeDisp(node, 1)) # DOF=1 (X direction)
            # Y-Y direction
            outputs2["shear"+str(i)].append(-op.eleForce(node, 2)) # DOF=2, displacement in Y
            outputs2["moment"+str(i)].append(-op.eleForce(node, 4)) # DOF=4, rotation in X axis (Yplane)
            outputs2["rel_dif_disp"+str(i)].append(op.nodeDisp(node+1, 2)-op.nodeDisp(node, 2)) # DOF=2 (Y direction)            
        
        for i in range(1, numStory+2):
            node = i
            # X-X direction
            outputs["disp"+str(i-1)].append(op.nodeDisp(node, 1))
            outputs["abs_accel"+str(i-1)].append(op.nodeAccel(node, 1)/9.8-motion[k]) # (node, 1)/9.8+motion[k])
            # Y-Y direction
            outputs2["disp"+str(i-1)].append(op.nodeDisp(node, 2))
            outputs2["abs_accel"+str(i-1)].append(op.nodeAccel(node, 2)/9.8-motion2[k]) # (node, 1)/9.8+motion2[k])
            
    # Save & export outputs
    for item in outputs:
        outputs[item] = np.array(outputs[item])
    for item in outputs2:
        outputs2[item] = np.array(outputs2[item])
   
    # # Save csv file
    # with open('NTHA.csv', 'w') as f:
    #     w = csv.DictWriter(f, outputs.keys())
    #     w.writeheader()
    #     w.writerow(outputs)
        
    if NumSimulations != 1 :
        GM_ID = GM_number
        plot_XX = 0
        plot_YY = 0
        plot_XY = 1
        direction = [plot_XX,plot_YY,plot_XY]
        plot_roof_drift(outputs, outputs2, direction, GM_ID)
    
    
    # Residual Drift
    
    #OPTION 1
    max_residual_drift = []
    max_residual_drift2 = []
    for i in range(1, numStory+1):
        Interstory_Drift = (outputs["disp"+str(i)][-1] - outputs["disp"+str(i-1)][-1]) / height
        max_residual_drift.append(abs(Interstory_Drift))
        Interstory_Drift2 = (outputs2["disp"+str(i)][-1] - outputs2["disp"+str(i-1)][-1]) / height
        max_residual_drift2.append(abs(Interstory_Drift2))
    outputs["residual_drift"] = max(max_residual_drift)
    outputs2["residual_drift"] = max(max_residual_drift2)   
    

    # #OPTION 2    
    # temp = 0.0
    # temp2 = 0.0
    # for i in range(1, numStory+1):
    #     temp = max(abs(outputs["rel_dif_disp"+str(i)][-1]),temp)
    #     temp2 = max(abs(outputs2["rel_dif_disp"+str(i)][-1]),temp2)
    # outputs["residual_drift"] = temp/height
    # outputs2["residual_drift"] = temp2/height
    
    
    
    return outputs, outputs2

	
# =============================================================================
# Plots
# =============================================================================

def plot_shear_backbone_curves(numStory, mat_prop, mat_prop2): # Plot Shear Force vs Displacement

    for i in range(numStory):  # Shear Spring
        mat_property = mat_prop["Shear_element"+str(i+1)]
        Vy = mat_property[0]
        Dy = mat_property[1]
        Vp = mat_property[2]
        Dp = mat_property[3]
        Du = mat_property[5]
        
        mat_property2 = mat_prop2["Shear_element"+str(i+1)]
        Vy2 = mat_property2[0]
        Dy2 = mat_property2[1]
        Vp2 = mat_property2[2]
        Dp2 = mat_property2[3]
        Du2 = mat_property2[5]
        
        sf = 1/1000 # 1/1000 for kN , 0.00022480894387096 for kips
        plt.plot([0, Dy, Dp, Du] , [0, Vy*sf, Vp*sf, Vp*sf], linewidth=1.0, label="NMFS E-W")
        plt.plot([0, Dy2, Dp2, Du2] , [0, Vy2*sf, Vp2*sf, Vp2*sf], linewidth=1.0, label="NMFS N-S")
       
        # plt.ylim([0, 5000])
        plt.xlim([0, 0.02])
        plt.xlabel('Displacement (m)')
        plt.ylabel('Shear Force (kN)')
        plt.title('Shear Backbone Curve for Level '+str(i+1))
        plt.legend()
        plt.grid()
        plt.show()          


def plot_flexural_backbone_curves(numStory, mat_prop, mat_prop2): # Plot Moment vs Rotation    
    for i in range(numStory):  # Flexural Spring
        mat_property = mat_prop["Flexural_element"+str(i+1)]
        My = mat_property[0]
        Ry = mat_property[1]
        Mp = mat_property[2]
        Rp = mat_property[3]
        Ru = mat_property[5]
        
        mat_property2 = mat_prop2["Flexural_element"+str(i+1)]
        My2 = mat_property2[0]
        Ry2 = mat_property2[1]
        Mp2 = mat_property2[2]
        Rp2 = mat_property2[3]
        Ru2 = mat_property2[5]
    
        sf = 1/1000 # 1/1000 for kN , 0.00022480894387096 for kips
        plt.plot([0, Ry, Rp, Ru] , [0, My*sf, Mp*sf, Mp*sf], linewidth=1.0, label="NMFS E-W")
        plt.plot([0, Ry2, Rp2, Ru2] , [0, My2*sf, Mp2*sf, Mp2*sf], linewidth=1.0, label="NMFS N-S")
        plt.xlim([0, 3.3/100]) # 0.00336 = 0.02/height
        plt.xlabel('Rotation (rad)')
        plt.ylabel('Moment (kN-m)')
        plt.title('Flexural Backbone Curve for Level '+str(i+1))
        plt.legend()
        plt.grid()
        plt.show()

def plot_displacements(outputs): # Plot Time vs Displacements
    plt.plot(outputs['time'], outputs["disp"+str(1)], outputs['time'], outputs["disp"+str(2)], outputs['time'], outputs["disp"+str(3)],
             outputs['time'], outputs["disp"+str(4)], outputs['time'], outputs["disp"+str(5)], outputs['time'], outputs["disp"+str(6)],
             outputs['time'], outputs["disp"+str(7)], outputs['time'], outputs["disp"+str(8)], outputs['time'], outputs["disp"+str(9)],
             outputs['time'], outputs["disp"+str(10)], outputs['time'], outputs["disp"+str(11)], outputs['time'], outputs["disp"+str(12)])
    plt.xlabel('Time (sec)')
    plt.ylabel('Story Displacement (m)')
    plt.title('Story Displacements')
    plt.grid()
    plt.show()    
    
def plot_roof_drift(outputs, outputs2, direction, GM_ID): # Plot Time vs Roof Drift

    # Plot X-X direction  
    if direction[0] == 1:
        H = outputs['numStory']*outputs['height']
        plt.plot(outputs['time'], outputs['rel_disp_roof']/H*100, linewidth=0.5, label="NMFS E-W")
        plt.xlabel('Time (sec)')
        plt.ylabel('Roof Drift (%)')
        plt.title('Roof Displacement - GM '+str(GM_ID))
        plt.legend()
        plt.grid()
        plt.show()
    
    # Plot Y-Y direction    
    if direction[1] == 1:
        H = outputs['numStory']*outputs['height']
        plt.plot(outputs2['time'], outputs2['rel_disp_roof']/H*100, linewidth=0.5, label="NMFS N-S")
        plt.xlabel('Time (sec)')
        plt.ylabel('Roof Drift (%)')
        plt.title('Roof Displacement - GM '+str(GM_ID))
        plt.legend()
        plt.grid()
        plt.show()
    
    # Plot both directions    
    if direction[2] == 1:
        H = outputs['numStory']*outputs['height']
        plt.plot(outputs['time'], outputs['rel_disp_roof']/H*100, linewidth=0.5, label="NMFS E-W")
        plt.plot(outputs2['time'], outputs2['rel_disp_roof']/H*100, linewidth=0.5, label="NMFS N-S")
        plt.xlabel('Time (sec)')
        plt.ylabel('Roof Drift (%)')
        plt.xlim([0, 500])
        plt.title('Roof Displacement - GM '+str(GM_ID))
        plt.legend()
        plt.grid()
        path = plot_results_path + '\\roof_drift_GM_'+str(GM_ID)
        plt.savefig(path, dpi =300)
        plt.show()    

def plot_story_shear(outputs, outputs2, direction): # Plot Maximum Story Shear
    
    numStory = outputs['numStory']
    Ygrid = []
    Max_Storey_Shears = []
    Max_Storey_Shears2 = []
    for i in range(1, numStory+1):
        Ygrid.append(i-1)
        Ygrid.append(i)
        Max_Storey_Shears.append(max(abs(outputs["shear"+str(i)]))/1000) # normalized
        Max_Storey_Shears.append(max(abs(outputs["shear"+str(i)]))/1000)
        Max_Storey_Shears2.append(max(abs(outputs2["shear"+str(i)]))/1000) # normalized
        Max_Storey_Shears2.append(max(abs(outputs2["shear"+str(i)]))/1000)
        
    # Plot X-X direction  
    if direction[0] == 1:    
        plt.plot(Max_Storey_Shears, Ygrid, linewidth=1.5, label="NMFS E-W")
        plt.xlabel('Max. Story Shear (kN)')
        plt.ylabel('Story')
        plt.title('Story Shears')
        plt.legend()
        plt.grid()
        plt.show()
    
    # Plot Y-Y direction  
    if direction[1] == 1:    
        plt.plot(Max_Storey_Shears2, Ygrid, linewidth=1.5, label="NMFS N-S")
        plt.xlabel('Max. Story Shear (kN)')
        plt.ylabel('Story')
        plt.title('Story Shears')
        plt.legend()
        plt.grid()
        plt.show()
    
    # Plot both directions  
    if direction[2] == 1:    
        plt.plot(Max_Storey_Shears, Ygrid, linewidth=1.5, label="NMFS E-W")
        plt.plot(Max_Storey_Shears2, Ygrid, linewidth=1.5, label="NMFS N-S")
        plt.xlabel('Max. Story Shear (kN)')
        plt.ylabel('Story')
        plt.title('Story Shears')
        plt.legend()
        plt.grid()
        plt.show()

def plot_story_shear_all(Outputs_list): # Plot Maximum Story Shear
      
    NumSimulations = int(len(Outputs_list)/2)
    numStory = Outputs_list[0]['numStory']
    EW_Shears = []
    NS_Shears = []
    
    # Get max TH story shears
    for GM_number in range(NumSimulations):
        outputs = Outputs_list[2*(GM_number)]           # E-W direction
        outputs2 = Outputs_list[2*(GM_number) + 1]      # N-S direction
        Ygrid = []
        Max_Storey_Shears = []
        Max_Storey_Shears2 = []
        for i in range(1, numStory+1):
            Ygrid.append(i-1)
            Ygrid.append(i)
            Max_Storey_Shears.append(max(abs(outputs["shear"+str(i)]))/1000000) # normalized
            Max_Storey_Shears.append(max(abs(outputs["shear"+str(i)]))/1000000)
            Max_Storey_Shears2.append(max(abs(outputs2["shear"+str(i)]))/1000000) # normalized
            Max_Storey_Shears2.append(max(abs(outputs2["shear"+str(i)]))/1000000)
        EW_Shears.append(Max_Storey_Shears)
        NS_Shears.append(Max_Storey_Shears2)
            
    # Plot X-X direction  
    mean_values = np.zeros(numStory*2)
    for GM_number in range(NumSimulations):
        Max_Storey_Shears = EW_Shears[GM_number]
        mean_values = np.add(mean_values, Max_Storey_Shears)
        plt.plot(Max_Storey_Shears, Ygrid, color = '0.5', linewidth=0.75) # label="GM"+str(GM_number+1)
        
    mean_values = mean_values/NumSimulations
    plt.plot(mean_values, Ygrid, color = '0', linewidth=2, label="Mean")
    plt.xlabel('Max. Story Shear (1e+3 kN)')
    plt.ylabel('Story')
    plt.title('Story Shears in E-W Direction')
    plt.legend()
    plt.grid()
    plt.xlim([0, 120])
    plt.ylim([0, 4])
    plt.yticks([0,1,2,3,4])
    path = plot_results_path+'\Story_Shears_E-W'
    plt.savefig(path, dpi =300)
    plt.show()
    
    # Plot Y-Y direction  
    mean_values2 = np.zeros(numStory*2)
    for GM_number in range(NumSimulations):
        Max_Storey_Shears2 = NS_Shears[GM_number]
        mean_values2 = np.add(mean_values2, Max_Storey_Shears2)
        plt.plot(Max_Storey_Shears2, Ygrid, color = '0.5', linewidth=0.75) # label="GM"+str(GM_number+1)
        
    mean_values2 = mean_values2/NumSimulations
    plt.plot(mean_values2, Ygrid, color = '0', linewidth=2, label="Mean")
    plt.xlabel('Max. Story Shear (1e+3 kN)')
    plt.ylabel('Story')
    plt.title('Story Shears in N-S Direction')
    plt.legend()
    plt.grid()
    plt.xlim([0, 120])
    plt.ylim([0, 4])
    plt.yticks([0,1,2,3,4])
    path = plot_results_path + '\Story_Shears_N-S'
    plt.savefig(path, dpi =300)
    plt.show()
        

def plot_hysteresis(Outputs_list): # Plot First Story Drift vs Base Shear
    
    NumSimulations = int(len(Outputs_list)/2)
    numStory = Outputs_list[0]['numStory']
    height = Outputs_list[0]['height']
    EW_Shears = []
    NS_Shears = []

    for GM_number in range(NumSimulations):
        outputs = Outputs_list[2*(GM_number)]           # E-W direction
        outputs2 = Outputs_list[2*(GM_number) + 1]      # N-S direction

    # Plot X-X direction   
        plt.plot(outputs['rel_disp_1story']/height*100, outputs['base_shear']/1000, linewidth=0.5, label="NMFS E-W")
        # plt.xlabel('1st InterStory Drift (%)')
        # plt.ylabel('Shear Force (kN)')
        # plt.title('Hysteresis Response - GM'+str(GM_number+1))
        # plt.legend()
        # plt.grid()
        # plt.show()
        
    # Plot Y-Y direction   
        plt.plot(outputs2['rel_disp_1story']/height*100, outputs2['base_shear']/1000, linewidth=0.5, label="NMFS N-S")
        plt.xlabel('1st InterStory Drift (%)')
        plt.ylabel('Shear Force (kN)')
        plt.title('Hysteresis Response (Shear) - GM'+str(GM_number+1))
        plt.legend()
        plt.grid()
        path = plot_results_path + '\Hysteresis_Shear_GM'+str(GM_number+1)
        plt.savefig(path, dpi =300)
        plt.show()
        
        
        plt.plot(outputs['rel_disp_1story']/height*100, outputs['moment1']/1000, linewidth=0.5, label="NMFS E-W") # moment at the base
        # plt.xlabel('1st InterStory Drift (%)')
        # plt.ylabel('Base Moment (kN-m)')
        # plt.title('Hysteresis Response')
        # plt.legend()
        # plt.grid()
        # plt.show()
        
        plt.plot(outputs2['rel_disp_1story']/height*100, -outputs2['moment1']/1000, linewidth=0.5, label="NMFS N-S") # moment at the base
        plt.xlabel('1st InterStory Drift (%)')
        plt.ylabel('Base Moment (kN-m)')
        plt.title('Hysteresis Response (Moment) - GM'+str(GM_number+1))
        plt.legend()
        plt.grid()
        path = plot_results_path + '\Hysteresis_Moment_GM'+str(GM_number+1)
        plt.savefig(path, dpi =300)
        plt.show() 
    
def plot_story_accel(outputs, outputs2, direction): # Plot Maximum Story Displacement
    
    numStory = outputs['numStory']
    Ygrid = []
    Max_Story_Accel = []
    Max_Story_Accel2 = []
    for i in range(1, numStory+2):
        Ygrid.append(i-1)
        Story_Accel = outputs["abs_accel"+str(i-1)][:]
        Max_Story_Accel.append(max(abs(Story_Accel))) # normalized
        Story_Accel2 = outputs2["abs_accel"+str(i-1)][:]
        Max_Story_Accel2.append(max(abs(Story_Accel2))) # normalized
    
    # Plot X-X direction  
    if direction[0] == 1:    
        plt.plot(Max_Story_Accel, Ygrid, linewidth=1.5, label="NMFS E-W")
        plt.xlabel('Acceleration (g)')
        plt.ylabel('Story')
        plt.title('Peak Floor Acceleration')
        plt.legend()
        plt.grid()
        plt.show()
    
    # Plot Y-Y direction  
    if direction[1] == 1:    
        plt.plot(Max_Story_Accel2, Ygrid, linewidth=1.5, label="NMFS N-S")
        plt.xlabel('Acceleration (g)')
        plt.ylabel('Story')
        plt.title('Peak Floor Acceleration')
        plt.legend()
        plt.grid()
        plt.show()

    # Plot both directions  
    if direction[2] == 1:    
        plt.plot(Max_Story_Accel, Ygrid, linewidth=1.5, label="NMFS E-W")
        plt.plot(Max_Story_Accel2, Ygrid, linewidth=1.5, label="NMFS N-S")
        plt.xlabel('Acceleration (g)')
        plt.ylabel('Story')
        plt.title('Peak Floor Acceleration')
        plt.legend()
        plt.grid()
        plt.show()
        
        
def plot_story_disp(outputs, outputs2, direction): # Plot Maximum Story Displacement
    
    numStory = outputs['numStory']
    Ygrid = []
    Max_Story_Disp = []
    Max_Story_Disp2 = []
    for i in range(1, numStory+2):
        Ygrid.append(i-1)
        Story_Disp = outputs["disp"+str(i-1)][:]
        Max_Story_Disp.append(max(abs(Story_Disp))) # normalized
        Story_Disp2 = outputs2["disp"+str(i-1)][:]
        Max_Story_Disp2.append(max(abs(Story_Disp2))) # normalized
    
    # Plot X-X direction  
    if direction[0] == 1:    
        plt.plot(Max_Story_Disp, Ygrid, linewidth=1.5, label="NMFS E-W")
        plt.xlabel('Max. Story Displacement (m)')
        plt.ylabel('Story')
        plt.title('Story Displacements')
        plt.legend()
        plt.grid()
        plt.show()
    
    # Plot Y-Y direction  
    if direction[1] == 1:    
        plt.plot(Max_Story_Disp2, Ygrid, linewidth=1.5, label="NMFS N-S")
        plt.xlabel('Max. Story Displacement (m)')
        plt.ylabel('Story')
        plt.title('Story Displacements')
        plt.legend()
        plt.grid()
        plt.show()

    # Plot both directions  
    if direction[2] == 1:    
        plt.plot(Max_Story_Disp, Ygrid, linewidth=1.5, label="NMFS E-W")
        plt.plot(Max_Story_Disp2, Ygrid, linewidth=1.5, label="NMFS N-S")
        plt.xlabel('Max. Story Displacement (m)')
        plt.ylabel('Story')
        plt.title('Story Displacements')
        plt.legend()
        plt.grid()
        plt.show()


def plot_story_accel_all(Outputs_list): # Plot Maximum Story Displacement

    
    NumSimulations = int(len(Outputs_list)/2)
    numStory = Outputs_list[0]['numStory']
    EW_Accel = []
    NS_Accel = []
    titles_font = {'fontname':'Arial'}
    
    # Get max TH storey accelerations
    for GM_number in range(NumSimulations):
        outputs = Outputs_list[2*(GM_number)]           # E-W direction
        outputs2 = Outputs_list[2*(GM_number) + 1]      # N-S direction
        Ygrid = []
        Max_Story_Accel = []
        Max_Story_Accel2 = []
        for i in range(1, numStory+2):
            Ygrid.append(i-1)
            Story_Accel = outputs["abs_accel"+str(i-1)][:]
            Max_Story_Accel.append(max(abs(Story_Accel))) # normalized
            Story_Accel2 = outputs2["abs_accel"+str(i-1)][:]
            Max_Story_Accel2.append(max(abs(Story_Accel2))) # normalized
        EW_Accel.append(Max_Story_Accel)
        NS_Accel.append(Max_Story_Accel2)

    # Plot X-X direction  
    mean_values = np.zeros(numStory+1)
    for GM_number in range(NumSimulations):
        Max_Story_Accel = EW_Accel[GM_number]
        mean_values = np.add(mean_values, Max_Story_Accel)
        plt.plot(Max_Story_Accel, Ygrid, color = '0.6', linewidth=1.5, alpha=0.3) # label="GM"+str(GM_number+1)
        
    mean_values = mean_values/NumSimulations
    plt.plot(mean_values, Ygrid, color = '0', linewidth=2.0, label="Mean")
    plt.plot([1,1], [0,4], color = '0.6', linewidth=1.5, alpha=0.3, label="Indiv. response" ) # just to put legend
    plt.xlabel('Floor Acceleration, g', fontsize=10, **titles_font)
    plt.ylabel('Stories', fontsize=10, **titles_font)
    # plt.title('Peak Floor Accelerations in E-W Direction', fontsize=10, **titles_font)
    leg = plt.legend(fontsize=8, loc="lower right")
    leg.get_frame().set_edgecolor('0.0')
    leg.get_frame().set_linewidth(0.75)
    # plt.grid()
    plt.xlim([0, 0.8])
    plt.ylim([0, 4])
    plt.xticks([0,0.2,0.4,0.6,0.8], fontsize=8)
    plt.yticks([0,1,2,3,4], fontsize=8)
    path = plot_results_path + '\Story_Accel_E-W'
    plt.savefig(path, bbox_inches="tight", dpi =300)
    plt.show()
        
    # Plot Y-Y direction  
    mean_values2 = np.zeros(numStory+1)
    for GM_number in range(NumSimulations):
        Max_Story_Accel2 = NS_Accel[GM_number]
        mean_values2 = np.add(mean_values2, Max_Story_Accel2)
        plt.plot(Max_Story_Accel2, Ygrid, color = '0.6', linewidth=1.5, alpha=0.3) # label="GM"+str(GM_number+1)
        
    mean_values2 = mean_values2/NumSimulations
    plt.plot(mean_values2, Ygrid, color = '0', linewidth=2.0, label="Mean")
    plt.plot([1,1], [0,4], color = '0.6', linewidth=1.5, alpha=0.3, label="Indiv. response" ) # just to put legend
    plt.xlabel('Floor Acceleration, g', fontsize=10, **titles_font)
    plt.ylabel('Stories', fontsize=10, **titles_font)
    # plt.title('Peak Floor Accelerations in N-S Direction', fontsize=10, **titles_font)
    leg = plt.legend(fontsize=8, loc="lower right")
    leg.get_frame().set_edgecolor('0.0')
    leg.get_frame().set_linewidth(0.75)
    # plt.grid()
    plt.xlim([0, 0.8])
    plt.ylim([0, 4])
    plt.xticks([0,0.2,0.4,0.6,0.8], fontsize=8)
    plt.yticks([0,1,2,3,4], fontsize=8)
    path = plot_results_path + '\Story_Accel_N-S'
    plt.savefig(path, bbox_inches="tight", dpi =300)
    plt.show()
    
    
    # Plot both directions
    
    for GM_number in range(NumSimulations):
        Max_Story_Accel = EW_Accel[GM_number]
        plt.plot(Max_Story_Accel, Ygrid, color = 'black', linewidth=1.2, alpha=0.075) # label="GM"+str(GM_number+1)
        Max_Story_Accel2 = NS_Accel[GM_number]
        plt.plot(Max_Story_Accel2, Ygrid, color = 'red', linewidth=1.2, alpha=0.075) # label="GM"+str(GM_number+1)
        
    plt.plot(mean_values, Ygrid, color = 'black', linewidth=2.0, label="E-W Mean")
    plt.plot(mean_values2, Ygrid, color = 'red', linewidth=2.0, label="N-S Mean ")
    
    
    plt.plot([1,1], [0,4], color = 'black', linewidth=1.2, alpha=0.1, label="E-W Indiv. Resp." ) # just to put legend
    plt.plot([1,1], [0,4], color = 'red', linewidth=1.2, alpha=0.1, label="N-S Indiv. Resp." ) # just to put legend
    
    plt.xlabel('Floor Acceleration, g', fontsize=10, **titles_font)
    plt.ylabel('Stories', fontsize=10, **titles_font)
    # plt.title('Peak Floor Accelerations in N-S Direction', fontsize=10, **titles_font)
    leg = plt.legend(fontsize=8, loc="lower right")
    leg.get_frame().set_edgecolor('0.0')
    leg.get_frame().set_linewidth(0.75)
    # plt.grid()
    plt.xlim([0, 1])
    plt.ylim([0, 4])
    plt.xticks([0,0.2,0.4,0.6,0.8,1.0], fontsize=8)
    plt.yticks([0,1,2,3,4], fontsize=8)
    path = plot_results_path + '\Story_Accel_All'
    plt.savefig(path, bbox_inches="tight", dpi =300)
    plt.show()    

    return EW_Accel, NS_Accel


def plot_story_disp_all(Outputs_list): # Plot Maximum Story Displacement

    
    NumSimulations = int(len(Outputs_list)/2)
    numStory = Outputs_list[0]['numStory']
    EW_Disps = []
    NS_Disps = []
    
    # Get max TH storey displacements
    for GM_number in range(NumSimulations):
        outputs = Outputs_list[2*(GM_number)]           # E-W direction
        outputs2 = Outputs_list[2*(GM_number) + 1]      # N-S direction
        Ygrid = []
        Max_Story_Disp = []
        Max_Story_Disp2 = []
        for i in range(1, numStory+2):
            Ygrid.append(i-1)
            Story_Disp = outputs["disp"+str(i-1)][:]
            Max_Story_Disp.append(max(abs(Story_Disp))) # normalized
            Story_Disp2 = outputs2["disp"+str(i-1)][:]
            Max_Story_Disp2.append(max(abs(Story_Disp2))) # normalized
        EW_Disps.append(Max_Story_Disp)
        NS_Disps.append(Max_Story_Disp2)

    # Plot X-X direction  
    mean_values = np.zeros(numStory+1)
    for GM_number in range(NumSimulations):
        Max_Story_Disp = EW_Disps[GM_number]
        mean_values = np.add(mean_values, Max_Story_Disp)
        plt.plot(Max_Story_Disp, Ygrid, color = '0.4', linewidth=2.0, alpha=0.3) # label="GM"+str(GM_number+1)
        
    mean_values = mean_values/NumSimulations
    plt.plot(mean_values, Ygrid, color = '0', linewidth=2.0, label="Mean")
    plt.xlabel('Max. Story Displacement (m)')
    plt.ylabel('Story')
    plt.title('Story Displacements in E-W Direction')
    plt.legend()
    plt.grid()
    plt.xlim([0, 0.11])
    plt.ylim([0, 4])
    plt.yticks([0,1,2,3,4])
    path = plot_results_path + '\Story_Disp_E-W'
    plt.savefig(path, bbox_inches="tight", dpi =300)
    plt.show()
        
    # Plot Y-Y direction  
    mean_values2 = np.zeros(numStory+1)
    for GM_number in range(NumSimulations):
        Max_Story_Disp2 = NS_Disps[GM_number]
        mean_values2 = np.add(mean_values2, Max_Story_Disp2)
        plt.plot(Max_Story_Disp2, Ygrid, color = '0.4', linewidth=2.0, alpha=0.3) # label="GM"+str(GM_number+1)
        
    mean_values2 = mean_values2/NumSimulations
    plt.plot(mean_values2, Ygrid, color = '0', linewidth=2.0, label="Mean")
    plt.xlabel('Max. Story Displacement (m)')
    plt.ylabel('Story')
    plt.title('Story Displacements in N-S Direction')
    plt.legend()
    plt.grid()
    plt.xlim([0, 0.031])
    plt.ylim([0, 4])
    plt.yticks([0,1,2,3,4])
    path = plot_results_path + '\Story_Disp_N-S'
    plt.savefig(path, bbox_inches="tight", dpi =300)
    plt.show()    

    return EW_Disps, NS_Disps
        
def plot_interstory_drift(outputs, outputs2, direction): # Plot Maximum Interstory Drift
    
    numStory = outputs['numStory']
    height = outputs['height']
    Ygrid = []
    Max_Interstory_Drifts = []
    Max_Interstory_Drifts2 = []
    for i in range(1, numStory+1):
        Ygrid.append(i)
        Interstory_Drift = (outputs["disp"+str(i)][:] - outputs["disp"+str(i-1)][:]) / height*100
        Max_Interstory_Drifts.append(max(abs(Interstory_Drift)))
        Interstory_Drift2 = (outputs2["disp"+str(i)][:] - outputs2["disp"+str(i-1)][:]) / height*100
        Max_Interstory_Drifts2.append(max(abs(Interstory_Drift2))) 
    
    # Plot X-X direction  
    if direction[0] == 1:  
        plt.plot(Max_Interstory_Drifts, Ygrid, linewidth=1.5, label="NMFS E-W")
        plt.xlabel('Max. Interstory Drift (%)')
        plt.ylabel('Story')
        plt.title('Interstory Drifts')
        plt.legend()
        plt.grid()
        plt.show()
        
    # Plot Y-Y direction  
    if direction[1] == 1:  
        plt.plot(Max_Interstory_Drifts2, Ygrid, linewidth=1.5, label="NMFS N-S")
        plt.xlabel('Max. Interstory Drift (%)')
        plt.ylabel('Story')
        plt.title('Interstory Drifts')
        plt.legend()
        plt.grid()
        plt.show()
    
    # Plot both directions  
    if direction[2] == 1:  
        plt.plot(Max_Interstory_Drifts, Ygrid, linewidth=1.5, label="NMFS E-W")
        plt.plot(Max_Interstory_Drifts2, Ygrid, linewidth=1.5, label="NMFS N-S")
        plt.xlabel('Max. Interstory Drift (%)')
        plt.ylabel('Story')
        plt.title('Interstory Drifts')
        plt.legend()
        plt.grid()
        plt.show()


def plot_interstory_drift_all(Outputs_list): # Plot Maximum Interstory Drift

    
    NumSimulations = int(len(Outputs_list)/2)
    numStory = Outputs_list[0]['numStory']
    height = Outputs_list[0]['height']
    EW_Drifts = []
    NS_Drifts = []
    
    titles_font = {'fontname':'Arial'}
    
    # Get max TH storey displacements
    for GM_number in range(NumSimulations):
        outputs = Outputs_list[2*(GM_number)]           # E-W direction
        outputs2 = Outputs_list[2*(GM_number) + 1]      # N-S direction
        Ygrid = []
        Max_Interstory_Drifts = []
        Max_Interstory_Drifts2 = []
        for i in range(1, numStory+1):
            Ygrid.append(i)
            Interstory_Drift = (outputs["disp"+str(i)][:] - outputs["disp"+str(i-1)][:]) / height
            Max_Interstory_Drifts.append(max(abs(Interstory_Drift)))
            Interstory_Drift2 = (outputs2["disp"+str(i)][:] - outputs2["disp"+str(i-1)][:]) / height
            Max_Interstory_Drifts2.append(max(abs(Interstory_Drift2))) 
        EW_Drifts.append(Max_Interstory_Drifts)
        NS_Drifts.append(Max_Interstory_Drifts2)

    # Plot X-X direction  
    mean_values = np.zeros(numStory)
    for GM_number in range(NumSimulations):
        Max_Interstory_Drifts = EW_Drifts[GM_number]
        mean_values = np.add(mean_values, Max_Interstory_Drifts)
        plt.plot([i * 100 for i in Max_Interstory_Drifts], Ygrid, color = '0.6', linewidth=1.5, alpha=0.3) # label="GM"+str(GM_number+1)
    mean_values = mean_values/NumSimulations
    
    plt.plot([i * 100 for i in mean_values], Ygrid, color = '0', linewidth=2.0, label="Mean")
    plt.plot(Ygrid, Ygrid, color = '0.6', linewidth=1.5, alpha=0.3, label="Indiv. response" ) # just to put legend
    plt.xlabel('Story Drift Ratio, %', fontsize=10, **titles_font)
    plt.ylabel('Stories', fontsize=10, **titles_font)
    # plt.title('Story Drifts in E-W Direction', fontsize=10, **titles_font)
    leg = plt.legend(fontsize=8, loc="lower right")
    leg.get_frame().set_edgecolor('0.0')
    leg.get_frame().set_linewidth(0.75)
    
    # plt.grid()
    plt.xlim([0, 0.5])
    plt.ylim([1, 4])
    plt.xticks([0,0.1,0.2,0.3,0.4,0.5], fontsize=8)
    plt.yticks([1,2,3,4], fontsize=8)
    # plt.tight_layout()
    path = plot_results_path + '\Inter-Story_Drift_E-W'
    plt.savefig(path, bbox_inches="tight", dpi =300)
    plt.show()

    # Plot Y-Y direction
    mean_values2 = np.zeros(numStory)
    for GM_number in range(NumSimulations):
        Max_Interstory_Drifts2 = NS_Drifts[GM_number]
        mean_values2 = np.add(mean_values2, Max_Interstory_Drifts2)
        plt.plot([i * 100 for i in Max_Interstory_Drifts2], Ygrid, color = '0.6', linewidth=1.5, alpha=0.3) # label="GM"+str(GM_number+1)
    mean_values2 = mean_values2/NumSimulations
    
    plt.plot([i * 100 for i in mean_values2], Ygrid, color = '0', linewidth=2.0, label="Mean")
    plt.plot(Ygrid, Ygrid, color = '0.6', linewidth=1.5, alpha=0.3, label="Indiv. response" ) # just to put legend
    plt.xlabel('Story Drift Ratio, %', fontsize=10, **titles_font)
    plt.ylabel('Stories', fontsize=10, **titles_font)
    # plt.title('Story Drifts in N-S Direction', fontsize=10, **titles_font)
    leg = plt.legend(fontsize=8, loc="lower right")
    leg.get_frame().set_edgecolor('0.0')
    leg.get_frame().set_linewidth(0.75)
    # plt.grid()
    plt.xlim([0, 0.3])
    plt.ylim([1, 4])
    plt.xticks([0,0.1,0.2,0.3], fontsize=8)
    plt.yticks([1,2,3,4], fontsize=8)
    # plt.tight_layout()
    path = plot_results_path + '\Inter-Story_Drift_N-S'
    plt.savefig(path, bbox_inches="tight", dpi =300)
    plt.show()
      
    # Plot both directions
      
    for GM_number in range(NumSimulations):
        Max_Interstory_Drifts = EW_Drifts[GM_number]
        plt.plot([i * 100 for i in Max_Interstory_Drifts], Ygrid, color = 'black', linewidth=1.2, alpha=0.05) # label="GM"+str(GM_number+1)
        Max_Interstory_Drifts2 = NS_Drifts[GM_number]
        plt.plot([i * 100 for i in Max_Interstory_Drifts2], Ygrid, color = 'red', linewidth=1.2, alpha=0.05) # label="GM"+str(GM_number+1)
    
    plt.plot([i * 100 for i in mean_values], Ygrid, color = 'black', linewidth=2.0, label="E-W Mean")
    plt.plot([i * 100 for i in mean_values2], Ygrid, color = 'red', linewidth=2.0, label="N-S Mean")
 
    plt.plot(Ygrid, Ygrid, color = 'black', linewidth=1.2, alpha=0.075, label="E-W Indiv. Resp" ) # just to put legend
    plt.plot(Ygrid, Ygrid, color = 'red', linewidth=1.2, alpha=0.075, label="N-S Indiv. Resp" ) # just to put legend
  
    plt.xlabel('Story Drift Ratio, %', fontsize=10, **titles_font)
    plt.ylabel('Stories', fontsize=10, **titles_font)
    # plt.title('Story Drifts in N-S Direction', fontsize=10, **titles_font)
    leg = plt.legend(fontsize=8, loc="lower right")
    leg.get_frame().set_edgecolor('0.0')
    leg.get_frame().set_linewidth(0.75)
    # plt.grid()
    plt.xlim([0, 0.5])
    plt.ylim([1, 4])
    plt.xticks([0,0.1,0.2,0.3,0.4,0.5], fontsize=8)
    plt.yticks([1,2,3,4], fontsize=8)
    
    # plt.tight_layout()
    path = plot_results_path + '\Inter-Story_Drift_All'
    plt.savefig(path, bbox_inches="tight", dpi =300)
    plt.show()    
    
    
    return EW_Drifts, NS_Drifts


def plot_spring_response(outputs, mat_prop, backbone_curve): # Plot First Story Drift vs Base Shear
    
    numStory = outputs['numStory']
    h = outputs['height']
        
    for i in range(1, numStory+1):  # Shear Spring
        plt.plot(outputs["rel_dif_disp"+str(i)], outputs["shear"+str(i)]/1000, linewidth=0.5, label="hysteresis")
        
        if backbone_curve == 1:
            bc = mat_prop["Shear_element"+str(i)]
            plt.plot([ 0,bc[1],bc[3],bc[5] ], [ 0,bc[0]/1000,bc[2]/1000,bc[4]/1000 ], linewidth=1., label="backbone curve")           
            plt.legend()
            
        plt.xlim([-0.1, 0.1])
        # plt.ylim([-10*1000, 10*1000])
        plt.xlabel('Displacement (m)')
        plt.ylabel('Shear Force (kN)')
        plt.title('Shear Spring Response for Level '+str(i))
        plt.grid()
        plt.show()
    
    for i in range(1, numStory+1):  # Flexural Spring
        plt.plot(outputs["rel_dif_disp"+str(i)], -outputs["moment"+str(i)]/1000, linewidth=0.5, label="hysteresis")
        
        if backbone_curve == 1:
            bc = mat_prop["Flexural_element"+str(i)]
            plt.plot([ 0,bc[1]*h,bc[3]*h,bc[5]*h ], [ 0,bc[0]/1000,bc[2]/1000,bc[4]/1000 ], linewidth=1., label="backbone curve")  
            plt.legend()
            
        plt.xlim([-0.1, 0.1])
        # plt.ylim([-120*1000, 120*1000])
        plt.xlabel('Displacement (m)')
        plt.ylabel('Moment (kN-m)')
        plt.title('Flexural Spring Response for Level '+str(i))
        plt.grid()
        plt.show()
       
 

# =============================================================================
# Pushover Analysis
# =============================================================================
    
def get_pushover(numStory, height, Umax, Nsteps, tol, modalMatrix, modalMatrix2):
    
    H = numStory*height

    ## X-X direction
    op.remove('loadPattern',1) # Remove load pattern
    op.remove('loadPattern',2) # Remove load pattern
    op.remove('loadPattern',3) # Remove load pattern
    op.reset() # Reset model to its initial state
    # Define the nonlinear static analysis
    load_tag_static = 4
    pattern_tag_static = 4
    
    op.timeSeries('Linear', load_tag_static)
    op.pattern('Plain', pattern_tag_static, load_tag_static)
    
    # Equal reference load on each floor (uniform vertical distribution of lateral loads)
    for i in range(1, numStory+1):
        # op.load(i+1,1,0.,0.)
        op.load(i+1,(modalMatrix[i,0])*1,0.,0.,0.,0.,0.) # 3d load!
        
    dU = Umax/Nsteps
    base_node = 1
    nodeControl = numStory+1
       
    op.wipeAnalysis()
    op.constraints('Plain')
    op.numberer('Plain')
    op.system('BandGeneral')
    op.test('NormDispIncr', tol, 100)
    op.algorithm('Newton')
    op.integrator('DisplacementControl',numStory+1, 1, dU) # Control roof displacement (DOF=1)
    op.analysis('Static')
    
    Uplot = op.nodeDisp(numStory+1,1)
    Pplot = op.getLoadFactor(pattern_tag_static)
    Mplot = op.getLoadFactor(pattern_tag_static)
    Step = []
    for i in range(Nsteps):
        ok = op.analyze(1)
        if ok < 0:
            break
        # Pplot = np.append(Pplot,op.getLoadFactor(pattern_tag_static))
        op.reactions()
        Step = np.append(Step,i)
        Pplot = np.append(Pplot,-op.nodeReaction(base_node, 1)) # DOF=1: X direction force
        Mplot = np.append(Mplot,op.nodeReaction(base_node, 5)) # DOF=5: rotation about Y axis (X plane moment)
        Uplot = np.append(Uplot,op.nodeDisp(nodeControl, 1))
       
        
    ## Y-Y direction
    op.remove('loadPattern',4) # Remove load pattern
    op.reset() # Reset model to its initial state
    # Define the nonlinear static analysis
    load_tag_static = 5
    pattern_tag_static = 5
    
    op.timeSeries('Linear', load_tag_static)
    op.pattern('Plain', pattern_tag_static, load_tag_static)
    
    # Equal reference load on each floor (uniform vertical distribution of lateral loads)
    for i in range(1, numStory+1):
        # op.load(i+1,1,0.,0.)
        op.load(i+1,0.,(modalMatrix2[i,0])*1, 0.,0.,0.,0.) # 3d load!
        
    dU = Umax/Nsteps
    base_node = 1
    nodeControl = numStory+1
       
    op.wipeAnalysis()
    op.constraints('Plain')
    op.numberer('Plain')
    op.system('BandGeneral')
    op.test('NormDispIncr', tol, 100)
    op.algorithm('Newton')  
    op.integrator('DisplacementControl',numStory+1, 2, dU) # Control roof displacement (DOF=2)
    op.analysis('Static')
    
    Uplot2 = op.nodeDisp(numStory+1,1)
    Pplot2 = op.getLoadFactor(pattern_tag_static)
    Mplot2 = op.getLoadFactor(pattern_tag_static)
    Step2 = []
    for i in range(Nsteps):
        ok = op.analyze(1)
        if ok < 0:
            break
        # Pplot = np.append(Pplot,op.getLoadFactor(pattern_tag_static))
        op.reactions()
        Step2 = np.append(Step2,i)
        Pplot2 = np.append(Pplot2,-op.nodeReaction(base_node, 2)) # DOF=2: Y direction force
        Mplot2 = np.append(Mplot2,-op.nodeReaction(base_node, 4)) # DOF=4: rotation about X axis (Y plane moment)
        Uplot2 = np.append(Uplot2,op.nodeDisp(nodeControl, 2))
        
    sf = 1/1000
    
    plt.plot(Uplot/H*100, Pplot*sf, linewidth=1.5, label="NMFS E-W")
    plt.plot(Uplot2/H*100, Pplot2*sf, linewidth=1.5, label="NMFS N-S")
    plt.xlabel('Roof Displacement (%)')
    plt.ylabel('Base Shear, kN')
    plt.title('Pushover Analysis')
    # plt.ylim([0, 5000])
    plt.xlim([0, 1])
    plt.legend()
    plt.grid()
    plt.show()
    
    plt.plot(Uplot/H*100, -Mplot*sf, linewidth=1.5, label="NMFS E-W")
    plt.plot(Uplot2/H*100, -Mplot2*sf, linewidth=1.5, label="NMFS N-S")
    plt.xlabel('Roof Displacement (%)')
    plt.ylabel('Base Moment, kN-m')
    plt.title('Pushover Analysis')
    # plt.ylim([0, 150000])
    plt.xlim([0, 1])
    plt.legend()
    plt.grid()
    plt.show()
    
    # outputs = {"step": [Step],"force": [Pplot],"rel_disp": [Uplot]} 
 
    # # Saves Pushover output in csv file
    # with open('Pushover.csv', 'w') as f:
    #     w = csv.DictWriter(f, outputs.keys())
    #     w.writeheader()
    #     w.writerow(outputs)

    return Step, Pplot, Uplot, Step2, Pplot2, Uplot2
    